@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -8,6 +8,8 @@ Maintainer: Itamar Shtull-Trauring
 """
 
 from zope.interface import Interface, Attribute
+from twisted.python.deprecate import deprecatedModuleAttribute
+from twisted.python.versions import Version
 
 
 class IAddress(Interface):
@@ -192,7 +194,17 @@ class IResolver(IResolverSimple):
         """
 
 
+
 class IReactorArbitrary(Interface):
+    """
+    This interface is redundant with L{IReactorFDSet} and is deprecated.
+    """
+    deprecatedModuleAttribute(
+        Version("Twisted", 10, 1, 0),
+        "See IReactorFDSet.",
+        __name__,
+        "IReactorArbitrary")
+
 
     def listenWith(portType, *args, **kw):
         """
@@ -206,6 +218,7 @@ class IReactorArbitrary(Interface):
         @return: an object which provides L{IListeningPort}.
         """
 
+
     def connectWith(connectorType, *args, **kw):
         """
         Start an instance of the given C{connectorType} connecting.
@@ -218,6 +231,13 @@ class IReactorArbitrary(Interface):
         @return:  An object which provides L{IConnector}.
         """
 
+# Alias for IReactorArbitrary so that internal Twisted code can continue to
+# provide the interface without emitting a deprecation warning.  This can be
+# removed when IReactorArbitrary is removed.
+_IReactorArbitrary = IReactorArbitrary
+
+
+
 class IReactorTCP(Interface):
 
     def listenTCP(port, factory, backlog=50, interface=''):
@@ -304,6 +324,7 @@ class IReactorSSL(Interface):
         """
 
 
+
 class IReactorUNIX(Interface):
     """
     UNIX socket methods.
@@ -321,11 +342,13 @@ class IReactorUNIX(Interface):
             has failed.
 
         @param checkPID: if True, check for a pid file to verify that a server
-            is listening.
+            is listening.  If C{address} is a Linux abstract namespace path,
+            this must be C{False}.
 
         @return: An object which provides L{IConnector}.
         """
 
+
     def listenUNIX(address, factory, backlog=50, mode=0666, wantPID=0):
         """
         Listen on a UNIX socket.
@@ -336,16 +359,19 @@ class IReactorUNIX(Interface):
 
         @param backlog: number of connections to allow in backlog.
 
-        @param mode: mode to set on the unix socket.  This parameter is
-            deprecated.  Permissions should be set on the directory which
-            contains the UNIX socket.
+        @param mode: The mode (B{not} umask) to set on the unix socket.  See
+            platform specific documentation for information about how this
+            might affect connection attempts.
+        @type mode: C{int}
 
-        @param wantPID: if True, create a pidfile for the socket.
+        @param wantPID: if True, create a pidfile for the socket.  If C{address}
+            is a Linux abstract namespace path, this must be C{False}.
 
         @return: An object which provides L{IListeningPort}.
         """
 
 
+
 class IReactorUNIXDatagram(Interface):
     """
     Datagram UNIX socket methods.
@@ -361,15 +387,17 @@ class IReactorUNIXDatagram(Interface):
 
         @param maxPacketSize: maximum packet size to accept
 
-        @param mode: mode to set on the unix socket.  This parameter is
-            deprecated.  Permissions should be set on the directory which
-            contains the UNIX socket.
+        @param mode: The mode (B{not} umask) to set on the unix socket.  See
+            platform specific documentation for information about how this
+            might affect connection attempts.
+        @type mode: C{int}
 
         @param bindAddress: address to bind to
 
         @return: An object which provides L{IConnector}.
         """
 
+
     def listenUNIXDatagram(address, protocol, maxPacketSize=8192, mode=0666):
         """
         Listen on a datagram UNIX socket.
@@ -380,14 +408,49 @@ class IReactorUNIXDatagram(Interface):
 
         @param maxPacketSize: maximum packet size to accept
 
-        @param mode: mode to set on the unix socket.  This parameter is
-            deprecated.  Permissions should be set on the directory which
-            contains the UNIX socket.
+        @param mode: The mode (B{not} umask) to set on the unix socket.  See
+            platform specific documentation for information about how this
+            might affect connection attempts.
+        @type mode: C{int}
 
         @return: An object which provides L{IListeningPort}.
         """
 
 
+
+class IReactorWin32Events(Interface):
+    """
+    Win32 Event API methods
+
+    @since: 10.2
+    """
+
+    def addEvent(event, fd, action):
+        """
+        Add a new win32 event to the event loop.
+
+        @param event: a Win32 event object created using win32event.CreateEvent()
+
+        @param fd: an instance of L{twisted.internet.abstract.FileDescriptor}
+
+        @param action: a string that is a method name of the fd instance.
+                       This method is called in response to the event.
+
+        @return: None
+        """
+
+
+    def removeEvent(event):
+        """
+        Remove an event.
+
+        @param event: a Win32 event object added using L{IReactorWin32Events.addEvent}
+
+        @return: None
+        """
+
+
+
 class IReactorUDP(Interface):
     """
     UDP socket methods.
@@ -417,13 +480,18 @@ class IReactorMulticast(Interface):
         L{DatagramProtocol<twisted.internet.protocol.DatagramProtocol>} to the
         given numeric UDP port.
 
-        @param listenMultiple: boolean indicating whether multiple sockets can
-                               bind to same UDP port.
+        @param listenMultiple: If set to True, allows multiple sockets to
+            bind to the same address and port number at the same time.
+        @type listenMultiple: C{bool}
 
         @returns: An object which provides L{IListeningPort}.
+
+        @see: L{twisted.internet.interfaces.IMulticastTransport}
+        @see: U{http://twistedmatrix.com/documents/current/core/howto/udp.html}
         """
 
 
+
 class IReactorProcess(Interface):
 
     def spawnProcess(processProtocol, executable, args=(), env={}, path=None,
@@ -542,17 +610,6 @@ class IReactorTime(Interface):
                  C{reset()} methods.
         """
 
-    def cancelCallLater(callID):
-        """
-        This method is deprecated.
-
-        Cancel a call that would happen later.
-
-        @param callID: this is an opaque identifier returned from C{callLater}
-                       that will be used to cancel a specific call.
-
-        @raise ValueError: if the callID is not recognized.
-        """
 
     def getDelayedCalls():
         """
@@ -1057,16 +1114,25 @@ class IConsumer(Interface):
         @return: C{None}
         """
 
+
     def unregisterProducer():
         """
         Stop consuming data from a producer, without disconnecting.
         """
 
+
     def write(data):
         """
         The producer will write data by calling this method.
+
+        The implementation must be non-blocking and perform whatever
+        buffering is necessary.  If the producer has provided enough data
+        for now and it is a L{IPushProducer}, the consumer may call its
+        C{pauseProducing} method.
         """
 
+
+
 class IFinishableConsumer(IConsumer):
     """
     A Consumer for producers that finish.
@@ -1510,7 +1576,7 @@ class IProcessTransport(ITransport):
         Send a signal to the process.
 
         @param signalID: can be
-          - one of C{\"HUP\"}, C{\"KILL\"}, C{\"STOP\"}, or C{\"INT\"}.
+          - one of C{"KILL"}, C{"TERM"}, or C{"INT"}.
               These will be implemented in a
               cross-platform manner, and so should be used
               if possible.
@@ -1589,21 +1655,6 @@ class IUDPTransport(Interface):
         """
 
 
-class IUDPConnectedTransport(Interface):
-    """
-    DEPRECATED. Transport for UDP ConnectedPacketProtocols.
-    """
-
-    def write(packet):
-        """
-        Write packet to address we are connected to.
-        """
-
-    def getHost():
-        """
-        Returns L{UNIXAddress}.
-        """
-
 
 class IUNIXDatagramTransport(Interface):
     """
@@ -1691,3 +1742,122 @@ class IMulticastTransport(Interface):
         """
         Leave multicast group, return L{Deferred} of success.
         """
+
+
+class IStreamClientEndpoint(Interface):
+    """
+    A stream client endpoint is a place that L{ClientFactory} can connect to.
+    For example, a remote TCP host/port pair would be a TCP client endpoint.
+
+    @since: 10.1
+    """
+
+    def connect(protocolFactory):
+        """
+        Connect the C{protocolFactory} to the location specified by this
+        L{IStreamClientEndpoint} provider.
+
+        @param protocolFactory: A provider of L{IProtocolFactory}
+        @return: A L{Deferred} that results in an L{IProtocol} upon successful
+            connection otherwise a L{ConnectError}
+        """
+
+
+
+class IStreamServerEndpoint(Interface):
+    """
+    A stream server endpoint is a place that a L{Factory} can listen for
+    incoming connections.
+
+    @since: 10.1
+    """
+
+    def listen(protocolFactory):
+        """
+        Listen with C{protocolFactory} at the location specified by this
+        L{IStreamServerEndpoint} provider.
+
+        @param protocolFactory: A provider of L{IProtocolFactory}
+        @return: A L{Deferred} that results in an L{IListeningPort} or an
+            L{CannotListenError}
+        """
+
+
+
+class IStreamServerEndpointStringParser(Interface):
+    """
+    An L{IStreamServerEndpointStringParser} is like an
+    L{IStreamClientEndpointStringParser}, except for L{IStreamServerEndpoint}s
+    instead of clients.  It integrates with L{endpoints.serverFromString} in
+    much the same way.
+    """
+
+    prefix = Attribute(
+        """
+        @see: L{IStreamClientEndpointStringParser.prefix}
+        """
+    )
+
+
+    def parseStreamServer(reactor, *args, **kwargs):
+        """
+        Parse a stream server endpoint from a reactor and string-only arguments
+        and keyword arguments.
+
+        @see: L{IStreamClientEndpointStringParser.parseStreamClient}
+
+        @return: a stream server endpoint
+        @rtype: L{IStreamServerEndpoint}
+        """
+
+
+
+class IStreamClientEndpointStringParser(Interface):
+    """
+    An L{IStreamClientEndpointStringParser} is a parser which can convert
+    a set of string C{*args} and C{**kwargs} into an L{IStreamClientEndpoint}
+    provider.
+
+    This interface is really only useful in the context of the plugin system
+    for L{endpoints.clientFromString}.  See the document entitled "I{The
+    Twisted Plugin System}" for more details on how to write a plugin.
+
+    If you place an L{IStreamClientEndpointStringParser} plugin in the
+    C{twisted.plugins} package, that plugin's C{parseStreamClient} method will
+    be used to produce endpoints for any description string that begins with
+    the result of that L{IStreamClientEndpointStringParser}'s prefix attribute.
+    """
+
+    prefix = Attribute(
+        """
+        A C{str}, the description prefix to respond to.  For example, an
+        L{IStreamClientEndpointStringParser} plugin which had C{"foo"} for its
+        C{prefix} attribute would be called for endpoint descriptions like
+        C{"foo:bar:baz"} or C{"foo:"}.
+        """
+    )
+
+
+    def parseStreamClient(reactor, *args, **kwargs):
+        """
+        This method is invoked by L{endpoints.clientFromString}, if the type of
+        endpoint matches the return value from this
+        L{IStreamClientEndpointStringParser}'s C{prefix} method.
+
+        @param reactor: The reactor passed to C{clientFromString}.
+
+        @param args: The string arguments, minus the endpoint type, in the
+            endpoint description string, parsed according to the rules
+            described in L{endpoints.quoteStringArgument}.  For example, if the
+            description were C{"my-type:foo:bar:baz=qux"}, C{args} would be
+            C{('foo','bar')}
+
+        @param kwargs: The string arguments from the endpoint description
+            passed as keyword arguments.  For example, if the description were
+            C{"my-type:foo:bar:baz=qux"}, C{kwargs} would be
+            C{dict(baz='qux')}.
+
+        @return: a client endpoint
+        @rtype: L{IStreamClientEndpoint}
+        """
+
