@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.test.test_process -*-
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -11,7 +11,7 @@ Maintainer: Itamar Shtull-Trauring
 """
 
 # System Imports
-import gc, os, sys, traceback, select, signal, errno
+import gc, os, sys, stat, traceback, select, signal, errno
 
 try:
     import pty
@@ -23,11 +23,14 @@ try:
 except ImportError:
     fcntl = None
 
+from zope.interface import implements
+
 from twisted.python import log, failure
 from twisted.python.util import switchUID
 from twisted.internet import fdesc, abstract, error
 from twisted.internet.main import CONNECTION_LOST, CONNECTION_DONE
 from twisted.internet._baseprocess import BaseProcess
+from twisted.internet.interfaces import IProcessTransport
 
 # Some people were importing this, which is incorrect, just keeping it
 # here for backwards compatibility:
@@ -106,6 +109,12 @@ class ProcessWriter(abstract.FileDescriptor):
 
     I am a helper which describes a selectable asynchronous writer to a
     process's input pipe, including stdin.
+
+    @ivar enableReadHack: A flag which determines how readability on this
+        write descriptor will be handled.  If C{True}, then readability may
+        indicate the reader for this write descriptor has been closed (ie,
+        the connection has been lost).  If C{False}, then readability events
+        are ignored.
     """
     connected = 1
     ic = 0
@@ -121,7 +130,12 @@ class ProcessWriter(abstract.FileDescriptor):
         self.name = name
         self.fd = fileno
 
-        if forceReadHack:
+        if not stat.S_ISFIFO(os.fstat(self.fileno()).st_mode):
+            # If the fd is not a pipe, then the read hack is never
+            # applicable.  This case arises when ProcessWriter is used by
+            # StandardIO and stdout is redirected to a normal file.
+            self.enableReadHack = False
+        elif forceReadHack:
             self.enableReadHack = True
         else:
             # Detect if this fd is actually a write-only fd. If it's
@@ -149,6 +163,9 @@ class ProcessWriter(abstract.FileDescriptor):
         """
         rv = fdesc.writeToFD(self.fd, data)
         if rv == len(data) and self.enableReadHack:
+            # If the send buffer is now empty and it is necessary to monitor
+            # this descriptor for readability to detect close, try detecting
+            # readability now.
             self.startReading()
         return rv
 
@@ -449,6 +466,137 @@ class _BaseProcess(BaseProcess, object):
         return "<%s pid=%s status=%s>" % (self.__class__.__name__,
                                           self.pid, self.status)
 
+
+class _FDDetector(object):
+    """
+    This class contains the logic necessary to decide which of the available
+    system techniques should be used to detect the open file descriptors for
+    the current process. The chosen technique gets monkey-patched into the
+    _listOpenFDs method of this class so that the detection only needs to occur
+    once.
+
+    @ivars listdir: The implementation of listdir to use. This gets overwritten
+        by the test cases.
+    @ivars getpid: The implementation of getpid to use, returns the PID of the
+        running process.
+    @ivars openfile: The implementation of open() to use, by default the Python
+        builtin.
+    """
+    # So that we can unit test this
+    listdir = os.listdir
+    getpid = os.getpid
+    openfile = open
+
+
+    def _listOpenFDs(self):
+        """
+        Figure out which implementation to use, then run it.
+        """
+        self._listOpenFDs = self._getImplementation()
+        return self._listOpenFDs()
+
+
+    def _getImplementation(self):
+        """
+        Check if /dev/fd works, if so, use that.  Otherwise, check if
+        /proc/%d/fd exists, if so use that.
+        
+        Otherwise, ask resource.getrlimit, if that throws an exception, then
+        fallback to _resourceFDImplementation or _fallbackFDImplementation as
+        last resort.
+        """
+        try:
+            self.listdir("/dev/fd")
+            if self._checkDevFDSanity(): # FreeBSD support :-)
+                return self._devFDImplementation
+        except:
+            try:
+                self.listdir("/proc/%d/fd" % (self.getpid(),))
+                return self._procFDImplementation
+            except:
+                pass
+
+        try:
+            self._resourceFDImplementation() # Imports resource
+            return self._resourceFDImplementation
+        except:
+            return self._fallbackFDImplementation
+
+    def _checkDevFDSanity(self):
+        """
+        Returns true iff opening a file modifies the fds visible
+        in /dev/fd, as it should on a sane platform.
+        """
+        start = self.listdir("/dev/fd")
+        fp = self.openfile("/dev/null", "r")
+        end = self.listdir("/dev/fd")
+        return start != end
+
+
+    def _devFDImplementation(self):
+        """
+        Simple implementation for systems where /dev/fd actually works.
+        See: http://www.freebsd.org/cgi/man.cgi?fdescfs
+        """
+        dname = "/dev/fd"
+        result = [int(fd) for fd in os.listdir(dname)]
+        return result
+
+
+    def _procFDImplementation(self):
+        """
+        Simple implementation for systems where /proc/pid/fd exists (we assume
+        it works).
+        """
+        dname = "/proc/%d/fd" % (os.getpid(),)
+        return [int(fd) for fd in os.listdir(dname)]
+
+
+    def _resourceFDImplementation(self):
+        """
+        Fallback implementation where the resource module can inform us about
+        how many FDs we can expect.
+
+        Note that on OS-X we expect to be using the /dev/fd implementation.
+        """
+        import resource
+        maxfds = resource.getrlimit(resource.RLIMIT_NOFILE)[0]
+        return xrange(maxfds)
+
+
+    def _fallbackFDImplementation(self):
+        """
+        Fallback-fallback implementation where we just assume that we need to
+        close 256 FDs.
+        """
+        maxfds = 256
+        return xrange(maxfds)
+
+
+detector = _FDDetector()
+
+def _listOpenFDs():
+    """
+    Use the global detector object to figure out which FD implementation to
+    use.
+    """
+
+    def _filterReallyOpenFDs(guess):
+        """
+        Remove false-positive from a list of supposedly open file descriptors.
+        
+        Returns a set of open file descriptors.
+        """
+        fds = set(guess)
+        poll = select.poll()
+        for fd in fds:
+            poll.register(fd, select.POLLIN)
+        return sorted(fds.difference(set(fd for (fd, ev) in poll.poll(0)
+                                  if ev == select.POLLNVAL)))
+
+    return _filterReallyOpenFDs(detector._listOpenFDs())
+
+
 class Process(_BaseProcess):
     """
     An operating-system Process.
@@ -462,6 +610,7 @@ class Process(_BaseProcess):
     code is not cross-platform. (also, windows can only select
     on sockets...)
     """
+    implements(IProcessTransport)
 
     debug = False
     debug_child = False
@@ -587,7 +736,8 @@ class Process(_BaseProcess):
         This is accomplished in two steps::
 
             1. close all file descriptors that aren't values of fdmap.  This
-               means 0 .. maxfds.
+               means 0 .. maxfds (or just the open fds within that range, if
+               the platform supports '/proc/<pid>/fd').
 
             2. for each childFD::
 
@@ -611,16 +761,7 @@ class Process(_BaseProcess):
             errfd.write("starting _setupChild\n")
 
         destList = fdmap.values()
-        try:
-            import resource
-            maxfds = resource.getrlimit(resource.RLIMIT_NOFILE)[1] + 1
-            # OS-X reports 9223372036854775808. That's a lot of fds to close
-            if maxfds > 1024:
-                maxfds = 1024
-        except:
-            maxfds = 256
-
-        for fd in xrange(maxfds):
+        for fd in _listOpenFDs():
             if fd in destList:
                 continue
             if debug and fd == errfd.fileno():
@@ -785,10 +926,13 @@ class Process(_BaseProcess):
         _BaseProcess.maybeCallProcessEnded(self)
 
 
+
 class PTYProcess(abstract.FileDescriptor, _BaseProcess):
     """
     An operating-system Process that uses PTY support.
     """
+    implements(IProcessTransport)
+
     status = -1
     pid = None
 
@@ -871,11 +1015,12 @@ class PTYProcess(abstract.FileDescriptor, _BaseProcess):
         os.dup2(slavefd, 1) # stdout
         os.dup2(slavefd, 2) # stderr
 
-        for fd in xrange(3, 256):
-            try:
-                os.close(fd)
-            except:
-                pass
+        for fd in _listOpenFDs():
+            if fd > 2:
+                try:
+                    os.close(fd)
+                except:
+                    pass
 
         self._resetSignalDisposition()
 
