@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.test.test_tcp -*-
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -18,10 +18,6 @@ import socket
 import sys
 import operator
 
-try:
-    import fcntl
-except ImportError:
-    fcntl = None
 from zope.interface import implements, classImplements
 
 try:
@@ -30,7 +26,7 @@ except ImportError:
     SSL = None
 
 from twisted.python.runtime import platformType
-
+from twisted.python import versions, deprecate
 
 if platformType == 'win32':
     # no such thing as WSAEPERM or error code 10001 according to winsock.h or MSDN
@@ -75,7 +71,8 @@ else:
 from errno import errorcode
 
 # Twisted Imports
-from twisted.internet import defer, base, address
+from twisted.internet import base, address, fdesc
+from twisted.internet.task import deferLater
 from twisted.python import log, failure, reflect
 from twisted.python.util import unsignedID
 from twisted.internet.error import CannotListenError
@@ -83,7 +80,7 @@ from twisted.internet import abstract, main, interfaces, error
 
 
 
-class _SocketCloser:
+class _SocketCloser(object):
     _socketShutdownMethod = 'shutdown'
 
     def _closeSocket(self):
@@ -460,14 +457,26 @@ class Connection(abstract.FileDescriptor, _SocketCloser):
                 return main.CONNECTION_LOST
         if not data:
             return main.CONNECTION_DONE
-        return self.protocol.dataReceived(data)
+        rval = self.protocol.dataReceived(data)
+        if rval is not None:
+            offender = self.protocol.dataReceived
+            warningFormat = (
+                'Returning a value other than None from %(fqpn)s is '
+                'deprecated since %(version)s.')
+            warningString = deprecate.getDeprecationWarningString(
+                offender, versions.Version('Twisted', 11, 0, 0),
+                format=warningFormat)
+            deprecate.warnAboutFunction(offender, warningString)
+        return rval
 
 
     def writeSomeData(self, data):
-        """Connection.writeSomeData(data) -> #of bytes written | CONNECTION_LOST
-        This writes as much data as possible to the socket and returns either
-        the number of bytes read (which is positive) or a connection error code
-        (which is negative)
+        """
+        Write as much as possible of the given data to this TCP connection.
+
+        This sends up to C{self.SEND_LIMIT} bytes from C{data}.  If the
+        connection is lost, an exception is returned.  Otherwise, the number
+        of bytes successfully written is returned.
         """
         try:
             # Limit length of buffer to try to send, because some OSes are too
@@ -600,9 +609,7 @@ class BaseClient(Connection):
         """
         s = socket.socket(self.addressFamily, self.socketType)
         s.setblocking(0)
-        if fcntl and hasattr(fcntl, 'FD_CLOEXEC'):
-            old = fcntl.fcntl(s.fileno(), fcntl.F_GETFD)
-            fcntl.fcntl(s.fileno(), fcntl.F_SETFD, old | fcntl.FD_CLOEXEC)
+        fdesc._setCloseOnExec(s.fileno())
         return s
 
     def resolveAddress(self):
@@ -711,14 +718,14 @@ class Client(BaseClient):
 
         This indicates the address from which I am connecting.
         """
-        return address.IPv4Address('TCP', *(self.socket.getsockname() + ('INET',)))
+        return address.IPv4Address('TCP', *self.socket.getsockname())
 
     def getPeer(self):
         """Returns an IPv4Address.
 
         This indicates the address that I am connected to.
         """
-        return address.IPv4Address('TCP', *(self.realAddress + ('INET',)))
+        return address.IPv4Address('TCP', *self.realAddress)
 
     def __repr__(self):
         s = '<%s to %s at %x>' % (self.__class__, self.addr, unsignedID(self))
@@ -773,14 +780,16 @@ class Server(Connection):
 
         This indicates the server's address.
         """
-        return address.IPv4Address('TCP', *(self.socket.getsockname() + ('INET',)))
+        return address.IPv4Address('TCP', *self.socket.getsockname())
 
     def getPeer(self):
         """Returns an IPv4Address.
 
         This indicates the client's address.
         """
-        return address.IPv4Address('TCP', *(self.client + ('INET',)))
+        return address.IPv4Address('TCP', *self.client)
+
+
 
 class Port(base.BasePort, _SocketCloser):
     """
@@ -929,6 +938,7 @@ class Port(base.BasePort, _SocketCloser):
                         break
                     raise
 
+                fdesc._setCloseOnExec(skt.fileno())
                 protocol = self.factory.buildProtocol(self._buildAddr(addr))
                 if protocol is None:
                     skt.close()
@@ -936,7 +946,6 @@ class Port(base.BasePort, _SocketCloser):
                 s = self.sessionno
                 self.sessionno = s+1
                 transport = self.transport(skt, protocol, addr, self, s, self.reactor)
-                transport = self._preMakeConnection(transport)
                 protocol.makeConnection(transport)
             else:
                 self.numberAccepts = self.numberAccepts+20
@@ -950,37 +959,37 @@ class Port(base.BasePort, _SocketCloser):
             # and return, so handling it here works just as well.
             log.deferr()
 
-    def _preMakeConnection(self, transport):
-        return transport
-
     def loseConnection(self, connDone=failure.Failure(main.CONNECTION_DONE)):
         """
         Stop accepting connections on this port.
 
         This will shut down the socket and call self.connectionLost().  It
         returns a deferred which will fire successfully when the port is
-        actually closed.
+        actually closed, or with a failure if an error occurs shutting down.
         """
         self.disconnecting = True
         self.stopReading()
         if self.connected:
-            self.deferred = defer.Deferred()
-            self.reactor.callLater(0, self.connectionLost, connDone)
+            self.deferred = deferLater(
+                self.reactor, 0, self.connectionLost, connDone)
             return self.deferred
 
     stopListening = loseConnection
 
 
+    def _logConnectionLostMsg(self):
+        """
+        Log message for closing port
+        """
+        log.msg('(TCP Port %s Closed)' % (self._realPortNumber,))
+
+
     def connectionLost(self, reason):
         """
         Cleans up the socket.
         """
-        log.msg('(Port %s Closed)' % self._realPortNumber)
+        self._logConnectionLostMsg()
         self._realPortNumber = None
-        d = None
-        if hasattr(self, "deferred"):
-            d = self.deferred
-            del self.deferred
 
         base.BasePort.connectionLost(self, reason)
         self.connected = False
@@ -990,16 +999,8 @@ class Port(base.BasePort, _SocketCloser):
 
         try:
             self.factory.doStop()
-        except:
+        finally:
             self.disconnecting = False
-            if d is not None:
-                d.errback(failure.Failure())
-            else:
-                raise
-        else:
-            self.disconnecting = False
-            if d is not None:
-                d.callback(None)
 
 
     def logPrefix(self):
@@ -1012,7 +1013,7 @@ class Port(base.BasePort, _SocketCloser):
 
         This indicates the server's address.
         """
-        return address.IPv4Address('TCP', *(self.socket.getsockname() + ('INET',)))
+        return address.IPv4Address('TCP', *self.socket.getsockname())
 
 class Connector(base.BaseConnector):
     def __init__(self, host, port, factory, timeout, bindAddress, reactor=None):
@@ -1030,4 +1031,4 @@ class Connector(base.BaseConnector):
         return Client(self.host, self.port, self.bindAddress, self, self.reactor)
 
     def getDestination(self):
-        return address.IPv4Address('TCP', self.host, self.port, 'INET')
+        return address.IPv4Address('TCP', self.host, self.port)
