@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.test.test_ftp -*-
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -252,9 +252,6 @@ class AnonUserDeniedError(FTPCmdError):
     Raised when an anonymous user issues a command that will alter the
     filesystem
     """
-    def __init__(self):
-        # No message
-        FTPCmdError.__init__(self, None)
 
     errorCode = ANON_USER_DENIED
 
@@ -914,7 +911,7 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
 
         try:
             segments = toSegments(self.workingDirectory, path)
-        except InvalidPath, e:
+        except InvalidPath:
             return defer.fail(FileNotFoundError(path))
 
         d = self.shell.list(
@@ -947,7 +944,7 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
 
         try:
             segments = toSegments(self.workingDirectory, path)
-        except InvalidPath, e:
+        except InvalidPath:
             return defer.fail(FileNotFoundError(path))
 
         def cbList(results):
@@ -1011,7 +1008,7 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
     def ftp_CWD(self, path):
         try:
             segments = toSegments(self.workingDirectory, path)
-        except InvalidPath, e:
+        except InvalidPath:
             # XXX Eh, what to fail with here?
             return defer.fail(FileNotFoundError(path))
 
@@ -1115,6 +1112,8 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
         def ebSent(err):
             log.msg("Unexpected error receiving file from client:")
             log.err(err)
+            if err.check(FTPCmdError):
+                return err
             return (CNX_CLOSED_TXFR_ABORTED,)
 
         def cbConsumer(cons):
@@ -1122,7 +1121,6 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
                 cons = ASCIIConsumerWrapper(cons)
 
             d = self.dtpInstance.registerConsumer(cons)
-            d.addCallbacks(cbSent, ebSent)
 
             # Tell them what to doooo
             if self.dtpInstance.isConnected:
@@ -1135,6 +1133,8 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
         def cbOpened(file):
             d = file.receive()
             d.addCallback(cbConsumer)
+            d.addCallback(lambda ignored: file.close())
+            d.addCallbacks(cbSent, ebSent)
             return d
 
         def ebOpened(err):
@@ -1507,7 +1507,16 @@ class IWriteFile(Interface):
         @rtype: C{Deferred} of C{IConsumer}
         """
 
+    def close():
+        """
+        Perform any post-write work that needs to be done. This method may
+        only be invoked once on each provider, and will always be invoked
+        after receive().
 
+        @rtype: C{Deferred} of anything: the value is ignored. The FTP client
+        will not see their upload request complete until this Deferred has
+        been fired.
+        """
 
 def _getgroups(uid):
     """Return the primary and supplementary groups for the given UID.
@@ -1621,13 +1630,23 @@ class FTPAnonymousShell(object):
 
 
     def openForReading(self, path):
+        """
+        Open C{path} for reading.
+
+        @param path: The path, as a list of segments, to open.
+        @type path: C{list} of C{unicode}
+        @return: A L{Deferred} is returned that will fire with an object
+            implementing L{IReadFile} if the file is successfully opened.  If
+            C{path} is a directory, or if an exception is raised while trying
+            to open the file, the L{Deferred} will fire with an error.
+        """
         p = self._path(path)
         if p.isdir():
             # Normally, we would only check for EISDIR in open, but win32
             # returns EACCES in this case, so we check before
             return defer.fail(IsADirectoryError(path))
         try:
-            f = p.open('rb')
+            f = p.open('r')
         except (IOError, OSError), e:
             return errnoToFailure(e.errno, path)
         except:
@@ -1840,13 +1859,23 @@ class FTPShell(FTPAnonymousShell):
 
 
     def openForWriting(self, path):
+        """
+        Open C{path} for writing.
+
+        @param path: The path, as a list of segments, to open.
+        @type path: C{list} of C{unicode}
+        @return: A L{Deferred} is returned that will fire with an object
+            implementing L{IWriteFile} if the file is successfully opened.  If
+            C{path} is a directory, or if an exception is raised while trying
+            to open the file, the L{Deferred} will fire with an error.
+        """
         p = self._path(path)
         if p.isdir():
             # Normally, we would only check for EISDIR in open, but win32
             # returns EACCES in this case, so we check before
             return defer.fail(IsADirectoryError(path))
         try:
-            fObj = p.open('wb')
+            fObj = p.open('w')
         except (IOError, OSError), e:
             return errnoToFailure(e.errno, path)
         except:
@@ -1868,28 +1897,91 @@ class _FileWriter(object):
         # FileConsumer will close the file object
         return defer.succeed(FileConsumer(self.fObj))
 
+    def close(self):
+        return defer.succeed(None)
 
 
-class FTPRealm:
+
+class BaseFTPRealm:
     """
-    @type anonymousRoot: L{twisted.python.filepath.FilePath}
-    @ivar anonymousRoot: Root of the filesystem to which anonymous
-    users will be granted access.
+    Base class for simple FTP realms which provides an easy hook for specifying
+    the home directory for each user.
     """
     implements(portal.IRealm)
 
     def __init__(self, anonymousRoot):
         self.anonymousRoot = filepath.FilePath(anonymousRoot)
 
+
+    def getHomeDirectory(self, avatarId):
+        """
+        Return a L{FilePath} representing the home directory of the given
+        avatar.  Override this in a subclass.
+
+        @param avatarId: A user identifier returned from a credentials checker.
+        @type avatarId: C{str}
+
+        @rtype: L{FilePath}
+        """
+        raise NotImplementedError(
+            "%r did not override getHomeDirectory" % (self.__class__,))
+
+
     def requestAvatar(self, avatarId, mind, *interfaces):
         for iface in interfaces:
             if iface is IFTPShell:
                 if avatarId is checkers.ANONYMOUS:
                     avatar = FTPAnonymousShell(self.anonymousRoot)
                 else:
-                    avatar = FTPShell(filepath.FilePath("/home/" + avatarId))
-                return IFTPShell, avatar, getattr(avatar, 'logout', lambda: None)
-        raise NotImplementedError("Only IFTPShell interface is supported by this realm")
+                    avatar = FTPShell(self.getHomeDirectory(avatarId))
+                return (IFTPShell, avatar,
+                        getattr(avatar, 'logout', lambda: None))
+        raise NotImplementedError(
+            "Only IFTPShell interface is supported by this realm")
+
+
+
+class FTPRealm(BaseFTPRealm):
+    """
+    @type anonymousRoot: L{twisted.python.filepath.FilePath}
+    @ivar anonymousRoot: Root of the filesystem to which anonymous
+        users will be granted access.
+
+    @type userHome: L{filepath.FilePath}
+    @ivar userHome: Root of the filesystem containing user home directories.
+    """
+    def __init__(self, anonymousRoot, userHome='/home'):
+        BaseFTPRealm.__init__(self, anonymousRoot)
+        self.userHome = filepath.FilePath(userHome)
+
+
+    def getHomeDirectory(self, avatarId):
+        """
+        Use C{avatarId} as a single path segment to construct a child of
+        C{self.userHome} and return that child.
+        """
+        return self.userHome.child(avatarId)
+
+
+
+class SystemFTPRealm(BaseFTPRealm):
+    """
+    L{SystemFTPRealm} uses system user account information to decide what the
+    home directory for a particular avatarId is.
+
+    This works on POSIX but probably is not reliable on Windows.
+    """
+    def getHomeDirectory(self, avatarId):
+        """
+        Return the system-defined home directory of the system user account with
+        the name C{avatarId}.
+        """
+        path = os.path.expanduser('~' + avatarId)
+        if path.startswith('~'):
+            raise cred_error.UnauthorizedLogin()
+        return filepath.FilePath(path)
+
+
 
 # --- FTP CLIENT  -------------------------------------------------------------
 
@@ -1950,8 +2042,23 @@ class ProtocolWrapper(protocol.Protocol):
         self.deferred.callback(None)
 
 
+
+class IFinishableConsumer(interfaces.IConsumer):
+    """
+    A Consumer for producers that finish.
+
+    @since: 11.0
+    """
+
+    def finish():
+        """
+        The producer has finished producing.
+        """
+
+
+
 class SenderProtocol(protocol.Protocol):
-    implements(interfaces.IFinishableConsumer)
+    implements(IFinishableConsumer)
 
     def __init__(self):
         # Fired upon connection
@@ -2684,7 +2791,7 @@ class FTPClient(FTPClientBasic):
                 # The only valid code is 257
                 if int(result[0].split(' ', 1)[0]) != 257:
                     raise ValueError
-            except (IndexError, ValueError), e:
+            except (IndexError, ValueError):
                 return failure.Failure(CommandFailed(result))
             path = parsePWDResponse(result[0])
             if path is None:
