@@ -1,8 +1,7 @@
 # -*- test-case-name: twisted.test.test_policies -*-
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
 """
 Resource limiting policies.
 
@@ -12,15 +11,24 @@ Resource limiting policies.
 # system imports
 import sys, operator
 
+from zope.interface import directlyProvides, providedBy
+
 # twisted imports
 from twisted.internet.protocol import ServerFactory, Protocol, ClientFactory
-from twisted.internet import reactor, error
+from twisted.internet import error
 from twisted.python import log
-from zope.interface import providedBy, directlyProvides
 
 
 class ProtocolWrapper(Protocol):
-    """Wraps protocol instances and acts as their transport as well."""
+    """
+    Wraps protocol instances and acts as their transport as well.
+
+    @ivar wrappedProtocol: An L{IProtocol} provider to which L{IProtocol}
+        method calls onto this L{ProtocolWrapper} will be proxied.
+
+    @ivar factory: The L{WrappingFactory} which created this
+        L{ProtocolWrapper}.
+    """
 
     disconnecting = 0
 
@@ -29,8 +37,15 @@ class ProtocolWrapper(Protocol):
         self.factory = factory
 
     def makeConnection(self, transport):
-        directlyProvides(self, *providedBy(self) + providedBy(transport))
+        """
+        When a connection is made, register this wrapper with its factory,
+        save the real transport, and connect the wrapped protocol to this
+        L{ProtocolWrapper} to intercept any transport calls it makes.
+        """
+        directlyProvides(self, providedBy(transport))
         Protocol.makeConnection(self, transport)
+        self.factory.registerProtocol(self)
+        self.wrappedProtocol.makeConnection(self)
 
     # Transport relaying
 
@@ -64,10 +79,6 @@ class ProtocolWrapper(Protocol):
 
     # Protocol relaying
 
-    def connectionMade(self):
-        self.factory.registerProtocol(self)
-        self.wrappedProtocol.makeConnection(self)
-
     def dataReceived(self, data):
         self.wrappedProtocol.dataReceived(data)
 
@@ -184,6 +195,7 @@ class ThrottlingFactory(WrappingFactory):
         """
         Wrapper around L{reactor.callLater} for test purpose.
         """
+        from twisted.internet import reactor
         return reactor.callLater(period, func)
 
 
@@ -481,6 +493,7 @@ class TimeoutFactory(WrappingFactory):
         """
         Wrapper around L{reactor.callLater} for test purpose.
         """
+        from twisted.internet import reactor
         return reactor.callLater(period, func)
 
 
@@ -582,7 +595,12 @@ class TrafficLoggingFactory(WrappingFactory):
 
 
 class TimeoutMixin:
-    """Mixin for protocols which wish to timeout connections
+    """
+    Mixin for protocols which wish to timeout connections.
+
+    Protocols that mix this in have a single timeout, set using L{setTimeout}.
+    When the timeout is hit, L{timeoutConnection} is called, which, by
+    default, closes the connection.
 
     @cvar timeOut: The number of seconds after which to timeout the connection.
     """
@@ -591,16 +609,31 @@ class TimeoutMixin:
     __timeoutCall = None
 
     def callLater(self, period, func):
+        """
+        Wrapper around L{reactor.callLater} for test purpose.
+        """
+        from twisted.internet import reactor
         return reactor.callLater(period, func)
 
 
     def resetTimeout(self):
-        """Reset the timeout count down"""
+        """
+        Reset the timeout count down.
+
+        If the connection has already timed out, then do nothing.  If the
+        timeout has been cancelled (probably using C{setTimeout(None)}), also
+        do nothing.
+
+        It's often a good idea to call this when the protocol has received
+        some meaningful input from the other end of the connection.  "I've got
+        some data, they're still there, reset the timeout".
+        """
         if self.__timeoutCall is not None and self.timeOut is not None:
             self.__timeoutCall.reset(self.timeOut)
 
     def setTimeout(self, period):
-        """Change the timeout period
+        """
+        Change the timeout period
 
         @type period: C{int} or C{NoneType}
         @param period: The period, in seconds, to change the timeout to, or
@@ -625,7 +658,9 @@ class TimeoutMixin:
         self.timeoutConnection()
 
     def timeoutConnection(self):
-        """Called when the connection times out.
+        """
+        Called when the connection times out.
+
         Override to define behavior other than dropping the connection.
         """
         self.transport.loseConnection()
