@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.test.test_log -*-
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -33,7 +33,7 @@ class ILogContext:
 class ILogObserver(Interface):
     """
     An observer which can do something with log events.
-    
+
     Given that most log observers are actually bound methods, it's okay to not
     explicitly declare provision of this interface.
     """
@@ -87,15 +87,7 @@ def callWithLogger(logger, func, *args, **kw):
     except:
         err(system=lp)
 
-def showwarning(message, category, filename, lineno, file=None, line=None):
-    if file is None:
-        msg(warning=message, category=reflect.qual(category), filename=filename, lineno=lineno,
-            format="%(filename)s:%(lineno)s: %(category)s: %(warning)s")
-    else:
-        if sys.version_info < (2, 6):
-            _oldshowwarning(message, category, filename, lineno, file)
-        else:
-            _oldshowwarning(message, category, filename, lineno, file, line)
+
 
 _keepErrors = 0
 _keptErrors = []
@@ -303,6 +295,28 @@ class LogPublisher:
                 self.observers[i] = observer
 
 
+    def showwarning(self, message, category, filename, lineno, file=None,
+                    line=None):
+        """
+        Twisted-enabled wrapper around L{warnings.showwarning}.
+
+        If C{file} is C{None}, the default behaviour is to emit the warning to
+        the log system, otherwise the original L{warnings.showwarning} Python
+        function is called.
+        """
+        if file is None:
+            self.msg(warning=message, category=reflect.qual(category),
+                     filename=filename, lineno=lineno,
+                     format="%(filename)s:%(lineno)s: %(category)s: %(warning)s")
+        else:
+            if sys.version_info < (2, 6):
+                _oldshowwarning(message, category, filename, lineno, file)
+            else:
+                _oldshowwarning(message, category, filename, lineno, file, line)
+
+
+
+
 try:
     theLogPublisher
 except NameError:
@@ -310,6 +324,7 @@ except NameError:
     addObserver = theLogPublisher.addObserver
     removeObserver = theLogPublisher.removeObserver
     msg = theLogPublisher.msg
+    showwarning = theLogPublisher.showwarning
 
 
 def _safeFormat(fmtString, fmtDict):
@@ -557,9 +572,16 @@ except NameError:
 def startLogging(file, *a, **kw):
     """
     Initialize logging to a specified file.
+
+    @return: A L{FileLogObserver} if a new observer is added, None otherwise.
     """
+    if isinstance(file, StdioOnnaStick):
+        return
     flo = FileLogObserver(file)
     startLoggingWithObserver(flo.emit, *a, **kw)
+    return flo
+
+
 
 def startLoggingWithObserver(observer, setStdout=1):
     """
@@ -612,15 +634,18 @@ class DefaultObserver:
     Will ignore all non-error messages and send error messages to sys.stderr.
     Will be removed when startLogging() is called for the first time.
     """
+    stderr = sys.stderr
 
     def _emit(self, eventDict):
         if eventDict["isError"]:
             if 'failure' in eventDict:
-                text = eventDict['failure'].getTraceback()
+                text = ((eventDict.get('why') or 'Unhandled Error')
+                        + '\n' + eventDict['failure'].getTraceback())
             else:
                 text = " ".join([str(m) for m in eventDict["message"]]) + "\n"
-            sys.stderr.write(text)
-            sys.stderr.flush()
+
+            self.stderr.write(text)
+            self.stderr.flush()
 
     def start(self):
         addObserver(self._emit)
