@@ -1,35 +1,40 @@
 # -*- test-case-name: twisted.test.test_reflect -*-
-
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
 """
 Standardized versions of various cool and/or strange things that you can do
 with Python's reflection capabilities.
 """
 
-from __future__ import nested_scopes
-
-
-# System Imports
 import sys
 import os
 import types
-import string
 import pickle
-import new
 import traceback
 import weakref
 import re
+import warnings
+import new
+try:
+    from collections import deque
+except ImportError:
+    deque = list
 
 RegexType = type(re.compile(""))
 
 
 try:
-    import cStringIO as StringIO
+    from cStringIO import StringIO
 except ImportError:
-    import StringIO
+    from StringIO import StringIO
+
+from twisted.python.util import unsignedID
+from twisted.python.deprecate import deprecated
+from twisted.python.deprecate import _fullyQualifiedName as fullyQualifiedName
+from twisted.python.versions import Version
+
+
 
 class Settable:
     """
@@ -86,16 +91,16 @@ class AccessorType(type):
                 if hasattr(self, name):
                     value = getattr(self, name)
                     def getter(this, value=value, name=name):
-                        if this.__dict__.has_key(name):
+                        if name in this.__dict__:
                             return this.__dict__[name]
                         else:
                             return value
                 else:
                     def getter(this, name=name):
-                        if this.__dict__.has_key(name):
+                        if name in this.__dict__:
                             return this.__dict__[name]
                         else:
-                            raise AttributeError, "no such attribute %r" % name
+                            raise AttributeError("no such attribute %r" % name)
             if setter is None:
                 def setter(this, value, name=name):
                     this.__dict__[name] = value
@@ -238,6 +243,7 @@ class Summer(Accessor):
                     Accessor.reallySet(self,y,v)
         Accessor.reallySet(self,k,v)
 
+
 class QueueMethod:
     """ I represent a method that doesn't exist yet."""
     def __init__(self, name, calls):
@@ -251,6 +257,10 @@ def funcinfo(function):
     """
     this is more documentation for myself than useful code.
     """
+    warnings.warn(
+        "[v2.5] Use inspect.getargspec instead of twisted.python.reflect.funcinfo",
+        DeprecationWarning,
+        stacklevel=2)
     code=function.func_code
     name=function.func_name
     argc=code.co_argcount
@@ -282,10 +292,12 @@ def fullFuncName(func):
         raise Exception("Couldn't find %s as %s." % (func, qualName))
     return qualName
 
+
 def qual(clazz):
     """Return full import path of a class."""
     return clazz.__module__ + '.' + clazz.__name__
 
+
 def getcurrent(clazz):
     assert type(clazz) == types.ClassType, 'must be a class...'
     module = namedModule(clazz.__module__)
@@ -294,6 +306,7 @@ def getcurrent(clazz):
         return clazz
     return currclass
 
+
 def getClass(obj):
     """Return the class or type of object 'obj'.
     Returns sensible result for oldstyle and newstyle instances and types."""
@@ -306,7 +319,7 @@ def getClass(obj):
 
 # I should really have a better name for this...
 def isinst(inst,clazz):
-    if type(inst) != types.InstanceType or type(clazz)!=types.ClassType:
+    if type(inst) != types.InstanceType or type(clazz)!= types.ClassType:
         return isinstance(inst,clazz)
     cl = inst.__class__
     cl2 = getcurrent(cl)
@@ -334,56 +347,141 @@ def namedModule(name):
 def namedObject(name):
     """Get a fully named module-global object.
     """
-    classSplit = string.split(name, '.')
-    module = namedModule(string.join(classSplit[:-1], '.'))
+    classSplit = name.split('.')
+    module = namedModule('.'.join(classSplit[:-1]))
     return getattr(module, classSplit[-1])
 
 namedClass = namedObject # backwards compat
 
-def namedAny(name):
-    """Get a fully named package, module, module-global object, or attribute.
+
+
+class _NoModuleFound(Exception):
     """
+    No module was found because none exists.
+    """
+
+
+class InvalidName(ValueError):
+    """
+    The given name is not a dot-separated list of Python objects.
+    """
+
+
+class ModuleNotFound(InvalidName):
+    """
+    The module associated with the given name doesn't exist and it can't be
+    imported.
+    """
+
+
+class ObjectNotFound(InvalidName):
+    """
+    The object associated with the given name doesn't exist and it can't be
+    imported.
+    """
+
+
+def _importAndCheckStack(importName):
+    """
+    Import the given name as a module, then walk the stack to determine whether
+    the failure was the module not existing, or some code in the module (for
+    example a dependent import) failing.  This can be helpful to determine
+    whether any actual application code was run.  For example, to distiguish
+    administrative error (entering the wrong module name), from programmer
+    error (writing buggy code in a module that fails to import).
+
+    @raise Exception: if something bad happens.  This can be any type of
+    exception, since nobody knows what loading some arbitrary code might do.
+
+    @raise _NoModuleFound: if no module was found.
+    """
+    try:
+        try:
+            return __import__(importName)
+        except ImportError:
+            excType, excValue, excTraceback = sys.exc_info()
+            while excTraceback:
+                execName = excTraceback.tb_frame.f_globals["__name__"]
+                if (execName is None or # python 2.4+, post-cleanup
+                    execName == importName): # python 2.3, no cleanup
+                    raise excType, excValue, excTraceback
+                excTraceback = excTraceback.tb_next
+            raise _NoModuleFound()
+    except:
+        # Necessary for cleaning up modules in 2.3.
+        sys.modules.pop(importName, None)
+        raise
+
+
+
+def namedAny(name):
+    """
+    Retrieve a Python object by its fully qualified name from the global Python
+    module namespace.  The first part of the name, that describes a module,
+    will be discovered and imported.  Each subsequent part of the name is
+    treated as the name of an attribute of the object specified by all of the
+    name which came before it.  For example, the fully-qualified name of this
+    object is 'twisted.python.reflect.namedAny'.
+
+    @type name: L{str}
+    @param name: The name of the object to return.
+
+    @raise InvalidName: If the name is an empty string, starts or ends with
+        a '.', or is otherwise syntactically incorrect.
+
+    @raise ModuleNotFound: If the name is syntactically correct but the
+        module it specifies cannot be imported because it does not appear to
+        exist.
+
+    @raise ObjectNotFound: If the name is syntactically correct, includes at
+        least one '.', but the module it specifies cannot be imported because
+        it does not appear to exist.
+
+    @raise AttributeError: If an attribute of an object along the way cannot be
+        accessed, or a module along the way is not found.
+
+    @return: the Python object identified by 'name'.
+    """
+    if not name:
+        raise InvalidName('Empty module name')
+
     names = name.split('.')
+
+    # if the name starts or ends with a '.' or contains '..', the __import__
+    # will raise an 'Empty module name' error. This will provide a better error
+    # message.
+    if '' in names:
+        raise InvalidName(
+            "name must be a string giving a '.'-separated list of Python "
+            "identifiers, not %r" % (name,))
+
     topLevelPackage = None
     moduleNames = names[:]
     while not topLevelPackage:
-        try:
+        if moduleNames:
             trialname = '.'.join(moduleNames)
-            topLevelPackage = __import__(trialname)
-        except ImportError:
-            # if the ImportError happened in the module being imported,
-            # this is a failure that should be handed to our caller.
-            # count stack frames to tell the difference.
-            exc_info = sys.exc_info()
-            if len(traceback.extract_tb(exc_info[2])) > 1:
-                try:
-                    # Clean up garbage left in sys.modules.
-                    del sys.modules[trialname]
-                except KeyError:
-                    # Python 2.4 has fixed this.  Yay!
-                    pass
-                raise exc_info[0], exc_info[1], exc_info[2]
-            moduleNames.pop()
-    
+            try:
+                topLevelPackage = _importAndCheckStack(trialname)
+            except _NoModuleFound:
+                moduleNames.pop()
+        else:
+            if len(names) == 1:
+                raise ModuleNotFound("No module named %r" % (name,))
+            else:
+                raise ObjectNotFound('%r does not name an object' % (name,))
+
     obj = topLevelPackage
     for n in names[1:]:
         obj = getattr(obj, n)
-        
+
     return obj
 
-def _reclass(clazz):
-    clazz = getattr(namedModule(clazz.__module__),clazz.__name__)
-    clazz.__bases__ = tuple(map(_reclass, clazz.__bases__))
-    return clazz
-
-
 
 
 def macro(name, filename, source, **identifiers):
     """macro(name, source, **identifiers)
 
-    This allows you to create macro-like behaviors in python.  See
-    twisted.python.hook for an example of its usage.
+    This allows you to create macro-like behaviors in python.
     """
     if not identifiers.has_key('name'):
         identifiers['name'] = name
@@ -416,6 +514,9 @@ def macro(name, filename, source, **identifiers):
 
     exec code in dict, dict
     return dict[name]
+macro = deprecated(Version("Twisted", 8, 2, 0))(macro)
+
+
 
 def _determineClass(x):
     try:
@@ -423,6 +524,8 @@ def _determineClass(x):
     except:
         return type(x)
 
+
+
 def _determineClassName(x):
     c = _determineClass(x)
     try:
@@ -431,61 +534,71 @@ def _determineClassName(x):
         try:
             return str(c)
         except:
-            return '<BROKEN CLASS AT %s>' % id(c)
+            return '<BROKEN CLASS AT 0x%x>' % unsignedID(c)
+
+
+
+def _safeFormat(formatter, o):
+    """
+    Helper function for L{safe_repr} and L{safe_str}.
+    """
+    try:
+        return formatter(o)
+    except:
+        io = StringIO()
+        traceback.print_exc(file=io)
+        className = _determineClassName(o)
+        tbValue = io.getvalue()
+        return "<%s instance at 0x%x with %s error:\n %s>" % (
+            className, unsignedID(o), formatter.__name__, tbValue)
+
+
 
 def safe_repr(o):
-    """safe_repr(anything) -> string
+    """
+    safe_repr(anything) -> string
 
     Returns a string representation of an object, or a string containing a
     traceback, if that object's __repr__ raised an exception.
     """
+    return _safeFormat(repr, o)
+
 
-    try:
-        return repr(o)
-    except:
-        io = StringIO.StringIO()
-        traceback.print_stack(file=io)
-        whati = _determineClassName(o)
-        swron = io.getvalue()
-        gwith = id(o)
-        you ='<%s instance at %s with repr error %s>' % (
-            whati,swron,gwith)
-        return you
 
 def safe_str(o):
-    """safe_str(anything) -> string
+    """
+    safe_str(anything) -> string
 
     Returns a string representation of an object, or a string containing a
     traceback, if that object's __str__ raised an exception.
     """
+    return _safeFormat(str, o)
 
-    try:
-        return str(o)
-    except:
-        strExc = '\n'.join(traceback.format_exception(*sys.exc_info()))
-        clsName = _determineClassName(o)
-        obId = id(o)
-        return '<%s instance at %s with str error %s>' % (
-            clsName, obId, strExc)
 
 
 ##the following were factored out of usage
 
+@deprecated(Version("Twisted", 11, 0, 0), "inspect.getmro")
 def allYourBase(classObj, baseClass=None):
     """allYourBase(classObj, baseClass=None) -> list of all base
     classes that are subclasses of baseClass, unless it is None,
     in which case all bases will be added.
     """
     l = []
-    accumulateBases(classObj, l, baseClass)
+    _accumulateBases(classObj, l, baseClass)
     return l
 
 
+@deprecated(Version("Twisted", 11, 0, 0), "inspect.getmro")
 def accumulateBases(classObj, l, baseClass=None):
+    _accumulateBases(classObj, l, baseClass)
+
+
+def _accumulateBases(classObj, l, baseClass=None):
     for base in classObj.__bases__:
         if baseClass is None or issubclass(base, baseClass):
             l.append(base)
-        accumulateBases(base, l, baseClass)
+        _accumulateBases(base, l, baseClass)
 
 
 def prefixedMethodNames(classObj, prefix):
@@ -518,6 +631,7 @@ def addMethodNamesToDict(classObj, dict, prefix, baseClass=None):
                 and (len(optName))):
                 dict[optName] = 1
 
+
 def prefixedMethods(obj, prefix=''):
     """A list of methods with a given prefix on a given instance.
     """
@@ -525,6 +639,7 @@ def prefixedMethods(obj, prefix=''):
     accumulateMethods(obj, dct, prefix)
     return dct.values()
 
+
 def accumulateMethods(obj, dict, prefix='', curClass=None):
     """accumulateMethods(instance, dict, prefix)
     I recurse through the bases of instance.__class__, and add methods
@@ -543,6 +658,7 @@ def accumulateMethods(obj, dict, prefix='', curClass=None):
             and (len(optName))):
             dict[optName] = getattr(obj, name)
 
+
 def accumulateClassDict(classObj, attr, adict, baseClass=None):
     """Accumulate all attributes of a given name in a class heirarchy into a single dictionary.
 
@@ -593,12 +709,15 @@ def accumulateClassList(classObj, attr, listObj, baseClass=None):
 def isSame(a, b):
     return (a is b)
 
+
 def isLike(a, b):
     return (a == b)
 
+
 def modgrep(goal):
     return objgrep(sys.modules, goal, isLike, 'sys.modules')
 
+
 def isOfType(start, goal):
     return ((type(start) is goal) or
             (isinstance(start, types.InstanceType) and
@@ -608,6 +727,7 @@ def isOfType(start, goal):
 def findInstances(start, t):
     return objgrep(start, t, isOfType)
 
+
 def objgrep(start, goal, eq=isLike, path='', paths=None, seen=None, showUnknowns=0, maxDepth=None):
     '''An insanely CPU-intensive process for finding stuff.
     '''
@@ -617,7 +737,7 @@ def objgrep(start, goal, eq=isLike, path='', paths=None, seen=None, showUnknowns
         seen = {}
     if eq(start, goal):
         paths.append(path)
-    if seen.has_key(id(start)):
+    if id(start) in seen:
         if seen[id(start)] is start:
             return
     if maxDepth is not None:
@@ -626,11 +746,10 @@ def objgrep(start, goal, eq=isLike, path='', paths=None, seen=None, showUnknowns
         maxDepth -= 1
     seen[id(start)] = start
     if isinstance(start, types.DictionaryType):
-        r = []
         for k, v in start.items():
             objgrep(k, goal, eq, path+'{'+repr(v)+'}', paths, seen, showUnknowns, maxDepth)
             objgrep(v, goal, eq, path+'['+repr(k)+']', paths, seen, showUnknowns, maxDepth)
-    elif isinstance(start, types.ListType) or isinstance(start, types.TupleType):
+    elif isinstance(start, (list, tuple, deque)):
         for idx in xrange(len(start)):
             objgrep(start[idx], goal, eq, path+'['+str(idx)+']', paths, seen, showUnknowns, maxDepth)
     elif isinstance(start, types.MethodType):
@@ -655,12 +774,10 @@ def objgrep(start, goal, eq=isLike, path='', paths=None, seen=None, showUnknowns
         print 'unknown type', type(start), start
     return paths
 
-def _startswith(s, sub):
-    # aug python2.1
-    return s[:len(sub)] == sub
 
 def filenameToModuleName(fn):
-    """Convert a name in the filesystem to the name of the Python module it is.
+    """
+    Convert a name in the filesystem to the name of the Python module it is.
 
     This is agressive about getting a module name back from a file; it will
     always return a string.  Agressive means 'sometimes wrong'; it won't look
@@ -669,7 +786,11 @@ def filenameToModuleName(fn):
     module.
     """
     fullName = os.path.abspath(fn)
-    modName = os.path.splitext(os.path.basename(fn))[0]
+    base = os.path.basename(fn)
+    if not base:
+        # this happens when fn ends with a path separator, just skit it
+        base = os.path.basename(fn[:-1])
+    modName = os.path.splitext(base)[0]
     while 1:
         fullName = os.path.dirname(fullName)
         if os.path.exists(os.path.join(fullName, "__init__.py")):
@@ -678,4 +799,20 @@ def filenameToModuleName(fn):
             break
     return modName
 
-#boo python
+
+
+__all__ = [
+    'InvalidName', 'ModuleNotFound', 'ObjectNotFound',
+
+    'ISNT', 'WAS', 'IS',
+
+    'Settable', 'AccessorType', 'PropertyAccessor', 'Accessor', 'Summer',
+    'QueueMethod', 'OriginalAccessor',
+
+    'funcinfo', 'fullFuncName', 'qual', 'getcurrent', 'getClass', 'isinst',
+    'namedModule', 'namedObject', 'namedClass', 'namedAny', 'macro',
+    'safe_repr', 'safe_str', 'allYourBase', 'accumulateBases',
+    'prefixedMethodNames', 'addMethodNamesToDict', 'prefixedMethods',
+    'accumulateClassDict', 'accumulateClassList', 'isSame', 'isLike',
+    'modgrep', 'isOfType', 'findInstances', 'objgrep', 'filenameToModuleName',
+    'fullyQualifiedName']
