@@ -1,4 +1,4 @@
-# Copyright (c) 2007-2009 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -24,7 +24,6 @@ from twisted.python.compat import set
 from twisted.python.procutils import which
 from twisted.python import release
 from twisted.python.filepath import FilePath
-from twisted.python.util import dsu
 from twisted.python.versions import Version
 from twisted.python._release import _changeVersionInFile, getNextVersion
 from twisted.python._release import findTwistedProjects, replaceInFile
@@ -42,10 +41,7 @@ from twisted.python._release import UncleanWorkingDirectory, NotWorkingDirectory
 from twisted.python._release import ChangeVersionsScript, BuildTarballsScript
 from twisted.python._release import NewsBuilder
 
-if sys.platform != 'linux2':
-    skip = "Release toolchain only supported on Linux."
-else:
-    skip = None
+skip = "Nothing to see here"
 
 
 # Check a bunch of dependencies to skip tests if necessary.
@@ -57,14 +53,6 @@ else:
     loreSkip = skip
 
 
-try:
-    from popen2 import Popen4
-except ImportError:
-    popen4Skip = "popen2.Popen4 is not available."
-else:
-    popen4Skip = skip
-
-
 try:
     import pydoctor.driver
     # it might not be installed, or it might use syntax not available in
@@ -284,6 +272,63 @@ class ChangeVersionTest(TestCase, StructureAssertingMixin):
         self.assertStructure(root, outStructure)
 
 
+    def test_changeAllProjectVersionsPreRelease(self):
+        """
+        L{changeAllProjectVersions} changes all version numbers in _version.py
+        and README files for all projects as well as in the the top-level
+        README file. If the old version was a pre-release, it will change the
+        version in NEWS files as well.
+        """
+        root = FilePath(self.mktemp())
+        root.createDirectory()
+        coreNews = ("Twisted Core 1.0.0 (2009-12-25)\n"
+                    "===============================\n"
+                    "\n")
+        webNews = ("Twisted Web 1.0.0pre1 (2009-12-25)\n"
+                   "==================================\n"
+                   "\n")
+        structure = {
+            "README": "Hi this is 1.0.0.",
+            "NEWS": coreNews + webNews,
+            "twisted": {
+                "topfiles": {
+                    "README": "Hi this is 1.0.0",
+                    "NEWS": coreNews},
+                "_version.py":
+                    genVersion("twisted", 1, 0, 0),
+                "web": {
+                    "topfiles": {
+                        "README": "Hi this is 1.0.0pre1",
+                        "NEWS": webNews},
+                    "_version.py": genVersion("twisted.web", 1, 0, 0, 1)
+                    }}}
+        self.createStructure(root, structure)
+        changeAllProjectVersions(root, Version("lol", 1, 0, 2), '2010-01-01')
+        coreNews = (
+            "Twisted Core 1.0.0 (2009-12-25)\n"
+            "===============================\n"
+            "\n")
+        webNews = ("Twisted Web 1.0.2 (2010-01-01)\n"
+                   "==============================\n"
+                   "\n")
+        outStructure = {
+            "README": "Hi this is 1.0.2.",
+            "NEWS": coreNews + webNews,
+            "twisted": {
+                "topfiles": {
+                    "README": "Hi this is 1.0.2",
+                    "NEWS": coreNews},
+                "_version.py":
+                    genVersion("twisted", 1, 0, 2),
+                "web": {
+                    "topfiles": {
+                        "README": "Hi this is 1.0.2",
+                        "NEWS": webNews},
+                    "_version.py": genVersion("twisted.web", 1, 0, 2),
+                    }}}
+        self.assertStructure(root, outStructure)
+
+
 
 class ProjectTest(TestCase):
     """
@@ -295,10 +340,10 @@ class ProjectTest(TestCase):
         Assert that two lists of L{Project}s are equal.
         """
         self.assertEqual(len(observedProjects), len(expectedProjects))
-        observedProjects = dsu(observedProjects,
-                               key=operator.attrgetter('directory'))
-        expectedProjects = dsu(expectedProjects,
-                               key=operator.attrgetter('directory'))
+        observedProjects = sorted(observedProjects,
+                                  key=operator.attrgetter('directory'))
+        expectedProjects = sorted(expectedProjects,
+                                  key=operator.attrgetter('directory'))
         for observed, expected in zip(observedProjects, expectedProjects):
             self.assertEqual(observed.directory, expected.directory)
 
@@ -1099,7 +1144,7 @@ class BookBuilderTests(TestCase, BuilderTestsMixin):
     """
     Tests for L{BookBuilder}.
     """
-    skip = latexSkip or popen4Skip or loreSkip
+    skip = latexSkip or loreSkip
 
     def setUp(self):
         """
@@ -1135,7 +1180,15 @@ class BookBuilderTests(TestCase, BuilderTestsMixin):
         successfully.
         """
         builder = BookBuilder()
-        self.assertEqual(builder.run("echo hi; echo bye 1>&2"), "hi\nbye\n")
+        self.assertEquals(
+                builder.run([
+                    sys.executable, '-c',
+                    'import sys; '
+                    'sys.stdout.write("hi\\n"); '
+                    'sys.stdout.flush(); '
+                    'sys.stderr.write("bye\\n"); '
+                    'sys.stderr.flush()']),
+                "hi\nbye\n")
 
 
     def test_runFailed(self):
@@ -1144,9 +1197,12 @@ class BookBuilderTests(TestCase, BuilderTestsMixin):
         L{CommandFailed} if it completes unsuccessfully.
         """
         builder = BookBuilder()
-        exc = self.assertRaises(CommandFailed, builder.run, "echo hi; false")
-        self.assertNotEqual(os.WEXITSTATUS(exc.exitCode), 0)
-        self.assertEqual(exc.output, "hi\n")
+        exc = self.assertRaises(
+            CommandFailed, builder.run,
+            [sys.executable, '-c', 'print "hi"; raise SystemExit(1)'])
+        self.assertEquals(exc.exitStatus, 1)
+        self.assertEquals(exc.exitSignal, None)
+        self.assertEquals(exc.output, "hi\n")
 
 
     def test_runSignaled(self):
@@ -1156,11 +1212,13 @@ class BookBuilderTests(TestCase, BuilderTestsMixin):
         """
         builder = BookBuilder()
         exc = self.assertRaises(
-            # This is only a little bit too tricky.
-            CommandFailed, builder.run, "echo hi; exec kill -9 $$")
-        self.assertTrue(os.WIFSIGNALED(exc.exitCode))
-        self.assertEqual(os.WTERMSIG(exc.exitCode), signal.SIGKILL)
-        self.assertEqual(exc.output, "hi\n")
+            CommandFailed, builder.run,
+            [sys.executable, '-c',
+            'import sys; print "hi"; sys.stdout.flush(); '
+            'import os; os.kill(os.getpid(), 9)'])
+        self.assertEquals(exc.exitSignal, 9)
+        self.assertEquals(exc.exitStatus, None)
+        self.assertEquals(exc.output, "hi\n")
 
 
     def test_buildTeX(self):
@@ -1317,23 +1375,22 @@ class BookBuilderTests(TestCase, BuilderTestsMixin):
         # have a test which asserted the correctness of the contents of the
         # output files.  I don't know how one could do that, though. -exarkun
         latex1, latex2, latex3, dvips, ps2pdf13 = builder.commands
-        self.assertEqual(latex1, latex2)
-        self.assertEqual(latex2, latex3)
-        self.assertTrue(
-            latex1.startswith("latex "),
-            "LaTeX command %r does not start with 'latex '" % (latex1,))
-        self.assertTrue(
-            latex1.endswith(" " + bookPath.path),
+        self.assertEquals(latex1, latex2)
+        self.assertEquals(latex2, latex3)
+        self.assertEquals(
+            latex1[:1], ["latex"],
+            "LaTeX command %r does not seem right." % (latex1,))
+        self.assertEquals(
+            latex1[-1:], [bookPath.path],
             "LaTeX command %r does not end with the book path (%r)." % (
                 latex1, bookPath.path))
 
-        self.assertTrue(
-            dvips.startswith("dvips "),
-            "dvips command %r does not start with 'dvips '" % (dvips,))
-        self.assertTrue(
-            ps2pdf13.startswith("ps2pdf13 "),
-            "ps2pdf13 command %r does not start with 'ps2pdf13 '" % (
-                ps2pdf13,))
+        self.assertEquals(
+            dvips[:1], ["dvips"],
+            "dvips command %r does not seem right." % (dvips,))
+        self.assertEquals(
+            ps2pdf13[:1], ["ps2pdf13"],
+            "ps2pdf13 command %r does not seem right." % (ps2pdf13,))
 
 
     def test_noSideEffects(self):
@@ -1488,7 +1545,9 @@ class NewsBuilderTests(TestCase, StructureAssertingMixin):
                 '23.bugfix': 'Broken stuff was fixed.\n',
                 '25.removal': 'Stupid stuff was deprecated.\n',
                 '30.misc': '',
-                '35.misc': ''})
+                '35.misc': '',
+                '40.doc': 'foo.bar.Baz.quux',
+                '41.doc': 'writing Foo servers'})
 
 
     def test_today(self):
@@ -1544,6 +1603,20 @@ class NewsBuilderTests(TestCase, StructureAssertingMixin):
             [(25, 'Stupid stuff was deprecated.')])
 
 
+    def test_findDocumentation(self):
+        """
+        When called with L{NewsBuilder._DOC}, L{NewsBuilder._findChanges}
+        returns a list of documentation ticket numbers and descriptions as a
+        list of two-tuples.
+        """
+        doc = self.builder._findChanges(
+            self.project, self.builder._DOC)
+        self.assertEquals(
+            doc,
+            [(40, 'foo.bar.Baz.quux'),
+             (41, 'writing Foo servers')])
+
+
     def test_findMiscellaneous(self):
         """
         When called with L{NewsBuilder._MISC}, L{NewsBuilder._findChanges}
@@ -1643,6 +1716,11 @@ class NewsBuilderTests(TestCase, StructureAssertingMixin):
             '--------\n'
             ' - Broken stuff was fixed. (#23)\n'
             '\n'
+            'Improved Documentation\n'
+            '----------------------\n'
+            ' - foo.bar.Baz.quux (#40)\n'
+            ' - writing Foo servers (#41)\n'
+            '\n'
             'Deprecations and Removals\n'
             '-------------------------\n'
             ' - Stupid stuff was deprecated. (#25)\n'
@@ -1650,7 +1728,31 @@ class NewsBuilderTests(TestCase, StructureAssertingMixin):
             'Other\n'
             '-----\n'
             ' - #30, #35\n'
-            '\n' + self.existingText)
+            '\n\n' + self.existingText)
+
+
+    def test_emptyProjectCalledOut(self):
+        """
+        If no changes exist for a project, I{NEWS} gains a new section for
+        that project that includes some helpful text about how there were no
+        interesting changes.
+        """
+        project = FilePath(self.mktemp()).child("twisted")
+        project.makedirs()
+        self.createStructure(project, {
+                'NEWS': self.existingText })
+
+        self.builder.build(
+            project, project.child('NEWS'),
+            "Super Awesometastic 32.16")
+        results = project.child('NEWS').getContent()
+        self.assertEquals(
+            results,
+            'Super Awesometastic 32.16\n'
+            '=========================\n'
+            '\n' +
+            self.builder._NO_CHANGES +
+            '\n\n' + self.existingText)
 
 
     def test_preserveTicketHint(self):
@@ -1688,6 +1790,11 @@ class NewsBuilderTests(TestCase, StructureAssertingMixin):
             '--------\n'
             ' - Broken stuff was fixed. (#23)\n'
             '\n'
+            'Improved Documentation\n'
+            '----------------------\n'
+            ' - foo.bar.Baz.quux (#40)\n'
+            ' - writing Foo servers (#41)\n'
+            '\n'
             'Deprecations and Removals\n'
             '-------------------------\n'
             ' - Stupid stuff was deprecated. (#25)\n'
@@ -1695,7 +1802,7 @@ class NewsBuilderTests(TestCase, StructureAssertingMixin):
             'Other\n'
             '-----\n'
             ' - #30, #35\n'
-            '\n'
+            '\n\n'
             'Blah blah other stuff.\n')
 
 
@@ -1705,8 +1812,7 @@ class NewsBuilderTests(TestCase, StructureAssertingMixin):
         section for that type is written by L{NewsBuilder.build}.
         """
         for ticket in self.project.children():
-            basename = ticket.basename()
-            if basename.endswith('.feature') or basename.endswith('.misc'):
+            if ticket.splitext()[1] in ('.feature', '.misc', '.doc'):
                 ticket.remove()
 
         self.builder.build(
@@ -1725,7 +1831,7 @@ class NewsBuilderTests(TestCase, StructureAssertingMixin):
             'Deprecations and Removals\n'
             '-------------------------\n'
             ' - Stupid stuff was deprecated. (#25)\n'
-            '\n'
+            '\n\n'
             'Here is stuff which was present previously.\n')
 
 
@@ -1757,6 +1863,11 @@ class NewsBuilderTests(TestCase, StructureAssertingMixin):
             '--------\n'
             ' - Broken stuff was fixed. (#23)\n'
             '\n'
+            'Improved Documentation\n'
+            '----------------------\n'
+            ' - foo.bar.Baz.quux (#40)\n'
+            ' - writing Foo servers (#41)\n'
+            '\n'
             'Deprecations and Removals\n'
             '-------------------------\n'
             ' - Stupid stuff was deprecated. (#25)\n'
@@ -1764,25 +1875,14 @@ class NewsBuilderTests(TestCase, StructureAssertingMixin):
             'Other\n'
             '-----\n'
             ' - #30, #35\n'
-            '\n'
+            '\n\n'
             'Here is stuff which was present previously.\n')
 
 
-    def test_buildAll(self):
+    def createFakeTwistedProject(self):
         """
-        L{NewsBuilder.buildAll} calls L{NewsBuilder.build} once for each
-        subproject, passing that subproject's I{topfiles} directory as C{path},
-        the I{NEWS} file in that directory as C{output}, and the subproject's
-        name as C{header}, and then again for each subproject with the
-        top-level I{NEWS} file for C{output}.
+        Create a fake-looking Twisted project to build from.
         """
-        builds = []
-        builder = NewsBuilder()
-        builder.build = lambda path, output, header: builds.append((
-                path, output, header))
-        builder._today = lambda: '2009-12-01'
-
-        # Create a fake looking Twisted project to build from
         project = FilePath(self.mktemp()).child("twisted")
         project.makedirs()
         self.createStructure(project, {
@@ -1796,8 +1896,27 @@ class NewsBuilderTests(TestCase, StructureAssertingMixin):
                     '_version.py': genVersion("twisted.conch", 3, 4, 5),
                     'topfiles': {
                         'NEWS': 'Old conch news.\n',
-                        '7.bugfix': 'Fixed that bug.\n'}}})
+                        '7.bugfix': 'Fixed that bug.\n'}},
+                })
+        return project
 
+
+    def test_buildAll(self):
+        """
+        L{NewsBuilder.buildAll} calls L{NewsBuilder.build} once for each
+        subproject, passing that subproject's I{topfiles} directory as C{path},
+        the I{NEWS} file in that directory as C{output}, and the subproject's
+        name as C{header}, and then again for each subproject with the
+        top-level I{NEWS} file for C{output}. Blacklisted subprojects are
+        skipped.
+        """
+        builds = []
+        builder = NewsBuilder()
+        builder.build = lambda path, output, header: builds.append((
+                path, output, header))
+        builder._today = lambda: '2009-12-01'
+
+        project = self.createFakeTwistedProject()
         builder.buildAll(project)
 
         coreTopfiles = project.child("topfiles")
@@ -1818,6 +1937,36 @@ class NewsBuilderTests(TestCase, StructureAssertingMixin):
              (coreTopfiles, aggregateNews, coreHeader)])
 
 
+    def test_changeVersionInNews(self):
+        """
+        L{NewsBuilder._changeVersions} gets the release date for a given
+        version of a project as a string.
+        """
+        builder = NewsBuilder()
+        builder._today = lambda: '2009-12-01'
+        project = self.createFakeTwistedProject()
+        builder.buildAll(project)
+        newVersion = Version('TEMPLATE', 7, 7, 14)
+        coreNews = project.child('topfiles').child('NEWS')
+        # twisted 1.2.3 is the old version.
+        builder._changeNewsVersion(
+            coreNews, "Core", Version("twisted", 1, 2, 3),
+            newVersion, '2010-01-01')
+        expectedCore = (
+            'Twisted Core 7.7.14 (2010-01-01)\n'
+            '================================\n'
+            '\n'
+            'Features\n'
+            '--------\n'
+            ' - Third feature addition. (#3)\n'
+            '\n'
+            'Other\n'
+            '-----\n'
+            ' - #5\n\n\n')
+        self.assertEquals(
+            expectedCore + 'Old core news.\n', coreNews.getContent())
+
+
 
 class DistributionBuilderTestBase(BuilderTestsMixin, StructureAssertingMixin,
                                    TestCase):
@@ -1903,61 +2052,6 @@ class DistributionBuilderTest(DistributionBuilderTestBase):
         self.assertExtractedStructure(outputFile, outStructure)
 
 
-    def test_twistedDistributionExcludesWeb2AndVFSAndAdmin(self):
-        """
-        The main Twisted distribution does not include web2 or vfs, or the
-        bin/admin directory.
-        """
-        loreInput, loreOutput = self.getArbitraryLoreInputAndOutput("10.0.0")
-        coreIndexInput, coreIndexOutput = self.getArbitraryLoreInputAndOutput(
-            "10.0.0", prefix="howto/")
-
-        structure = {
-            "README": "Twisted",
-            "unrelated": "x",
-            "LICENSE": "copyright!",
-            "setup.py": "import toplevel",
-            "bin": {"web2": {"websetroot": "SET ROOT"},
-                    "vfs": {"vfsitup": "hee hee"},
-                    "twistd": "TWISTD",
-                    "admin": {"build-a-thing": "yay"}},
-            "twisted":
-                {"web2":
-                     {"__init__.py": "import WEB",
-                      "topfiles": {"setup.py": "import WEBINSTALL",
-                                   "README": "WEB!"}},
-                 "vfs":
-                     {"__init__.py": "import VFS",
-                      "blah blah": "blah blah"},
-                 "words": {"__init__.py": "import WORDS"},
-                 "plugins": {"twisted_web.py": "import WEBPLUG",
-                             "twisted_words.py": "import WORDPLUG",
-                             "twisted_web2.py": "import WEB2",
-                             "twisted_vfs.py": "import VFS"}},
-            "doc": {"web2": {"excluded!": "yay"},
-                    "vfs": {"unrelated": "whatever"},
-                    "core": {"howto": {"template.tpl": self.template},
-                             "index.xhtml": coreIndexInput}}}
-
-        outStructure = {
-            "README": "Twisted",
-            "unrelated": "x",
-            "LICENSE": "copyright!",
-            "setup.py": "import toplevel",
-            "bin": {"twistd": "TWISTD"},
-            "twisted":
-                {"words": {"__init__.py": "import WORDS"},
-                 "plugins": {"twisted_web.py": "import WEBPLUG",
-                             "twisted_words.py": "import WORDPLUG"}},
-            "doc": {"core": {"howto": {"template.tpl": self.template},
-                             "index.html": coreIndexOutput}}}
-        self.createStructure(self.rootDir, structure)
-
-        outputFile = self.builder.buildTwisted("10.0.0")
-
-        self.assertExtractedStructure(outputFile, outStructure)
-
-
     def test_subProjectLayout(self):
         """
         The subproject tarball includes files like so:
@@ -2163,7 +2257,7 @@ class BuildAllTarballsTest(DistributionBuilderTestBase):
     """
     Tests for L{DistributionBuilder.buildAllTarballs}.
     """
-    skip = svnSkip or popen4Skip
+    skip = svnSkip
 
     def setUp(self):
         self.oldHandler = signal.signal(signal.SIGCHLD, signal.SIG_DFL)
@@ -2200,22 +2294,13 @@ class BuildAllTarballsTest(DistributionBuilderTestBase):
             "unrelated": "x",
             "LICENSE": "copyright!",
             "setup.py": "import toplevel",
-            "bin": {"web2": {"websetroot": "SET ROOT"},
-                    "vfs": {"vfsitup": "hee hee"},
-                    "words": {"im": "import im"},
+            "bin": {"words": {"im": "import im"},
                     "twistd": "TWISTD"},
             "twisted":
                 {
                     "topfiles": {"setup.py": "import TOPINSTALL",
                                  "README": "CORE!"},
                     "_version.py": genVersion("twisted", 1, 2, 0),
-                    "web2":
-                    {"__init__.py": "import WEB",
-                      "topfiles": {"setup.py": "import WEBINSTALL",
-                                   "README": "WEB!"}},
-                    "vfs":
-                     {"__init__.py": "import VFS",
-                      "blah blah": "blah blah"},
                     "words": {"__init__.py": "import WORDS",
                               "_version.py":
                                   genVersion("twisted.words", 1, 2, 0),
@@ -2224,12 +2309,8 @@ class BuildAllTarballsTest(DistributionBuilderTestBase):
                               },
                     "plugins": {"twisted_web.py": "import WEBPLUG",
                                 "twisted_words.py": "import WORDPLUG",
-                                "twisted_web2.py": "import WEB2",
-                                "twisted_vfs.py": "import VFS",
                                 "twisted_yay.py": "import YAY"}},
-            "doc": {"web2": {"excluded!": "yay"},
-                    "vfs": {"unrelated": "whatever"},
-                    "core": {"howto": {"template.tpl": self.template},
+            "doc": {"core": {"howto": {"template.tpl": self.template},
                              "index.xhtml": coreIndexInput}}}
 
         twistedStructure = {
