@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -557,3 +557,28 @@ class PythonLoggingIntegrationTestCase(unittest.TestCase):
         finally:
             log.PythonLoggingObserver.emit = oldEmit
 
+
+class DefaultObserverTestCase(unittest.TestCase):
+    """
+    Test the default observer.
+    """
+
+    def test_failureLogger(self):
+        """
+        The reason argument passed to log.err() appears in the report
+        generated by DefaultObserver.
+        """
+        from StringIO import StringIO
+        
+        obs = log.DefaultObserver()
+        obs.stderr = StringIO()
+        obs.start()
+
+        reason = "The reason."
+        log.err(Exception(), reason)
+        errors = self.flushLoggedErrors()
+
+        self.assertSubstring(reason, obs.stderr.getvalue())
+        self.assertEquals(len(errors), 1)
+
+        obs.stop()
