@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -21,6 +21,7 @@ class AbstractFilePathTestCase(unittest.TestCase):
     f1content = "file 1"
     f2content = "file 2"
 
+
     def _mkpath(self, *p):
         x = os.path.abspath(os.path.join(self.cmn, *p))
         self.all.append(x)
@@ -136,6 +137,16 @@ class AbstractFilePathTestCase(unittest.TestCase):
         self.failUnlessEqual(f2.open().read(), self.f2content)
 
 
+    def test_multipleChildSegments(self):
+        """
+        C{fp.descendant([a, b, c])} returns the same L{FilePath} as is returned
+        by C{fp.child(a).child(b).child(c)}.
+        """
+        multiple = self.path.descendant(['a', 'b', 'c'])
+        single = self.path.child('a').child('b').child('c')
+        self.assertEquals(multiple, single)
+
+
     def test_dictionaryKeys(self):
         """
         Verify that path instances are usable as dictionary keys.
@@ -265,8 +276,7 @@ def zipit(dirname, zfname):
     create a zipfile on zfname, containing the contents of dirname'
     """
     zf = zipfile.ZipFile(zfname, "w")
-    basedir = os.path.basename(dirname)
-    for root, dirs, files, in os.walk(dirname):
+    for root, ignored, files, in os.walk(dirname):
         for fname in files:
             fspath = os.path.join(root, fname)
             arcpath = os.path.join(root, fname)[len(dirname)+1:]
@@ -310,6 +320,29 @@ class ZipFilePathTestCase(AbstractFilePathTestCase):
         self.assertEquals(repr(child), pathRepr)
 
 
+    def test_zipPathReprParentDirSegment(self):
+        """
+        The repr of a ZipPath with C{".."} in the internal part of its path
+        includes the C{".."} rather than applying the usual parent directory
+        meaning.
+        """
+        child = self.path.child("foo").child("..").child("bar")
+        pathRepr = "ZipPath(%r)" % (
+            self.cmn + ".zip" + os.sep.join(["", "foo", "..", "bar"]))
+        self.assertEquals(repr(child), pathRepr)
+
+
+    def test_zipPathReprEscaping(self):
+        """
+        Bytes in the ZipPath path which have special meaning in Python
+        string literals are escaped in the ZipPath repr.
+        """
+        child = self.path.child("'")
+        path = self.cmn + ".zip" + os.sep.join(["", "'"])
+        pathRepr = "ZipPath('%s')" % (path.encode('string-escape'),)
+        self.assertEquals(repr(child), pathRepr)
+
+
     def test_zipArchiveRepr(self):
         """
         Make sure that invoking ZipArchive's repr prints the correct class
@@ -329,7 +362,139 @@ class ZipFilePathTestCase(AbstractFilePathTestCase):
 
 
 
+class ExplodingFile:
+    """
+    A C{file}-alike which raises exceptions from its I/O methods and keeps track
+    of whether it has been closed.
+
+    @ivar closed: A C{bool} which is C{False} until C{close} is called, then it
+        is C{True}.
+    """
+    closed = False
+
+    def read(self, n=0):
+        """
+        @raise IOError: Always raised.
+        """
+        raise IOError()
+
+
+    def write(self, what):
+        """
+        @raise IOError: Always raised.
+        """
+        raise IOError()
+
+
+    def close(self):
+        """
+        Mark the file as having been closed.
+        """
+        self.closed = True
+
+
+
+class TrackingFilePath(filepath.FilePath):
+    """
+    A subclass of L{filepath.FilePath} which maintains a list of all other paths
+    created by clonePath.
+
+    @ivar trackingList: A list of all paths created by this path via
+        C{clonePath} (which also includes paths created by methods like
+        C{parent}, C{sibling}, C{child}, etc (and all paths subsequently created
+        by those paths, etc).
+
+    @type trackingList: C{list} of L{TrackingFilePath}
+
+    @ivar openedFiles: A list of all file objects opened by this
+        L{TrackingFilePath} or any other L{TrackingFilePath} in C{trackingList}.
+
+    @type openedFiles: C{list} of C{file}
+    """
+
+    def __init__(self, path, alwaysCreate=False, trackingList=None):
+        filepath.FilePath.__init__(self, path, alwaysCreate)
+        if trackingList is None:
+            trackingList = []
+        self.trackingList = trackingList
+        self.openedFiles = []
+
+
+    def open(self, *a, **k):
+        """
+        Override 'open' to track all files opened by this path.
+        """
+        f = filepath.FilePath.open(self, *a, **k)
+        self.openedFiles.append(f)
+        return f
+
+
+    def openedPaths(self):
+        """
+        Return a list of all L{TrackingFilePath}s associated with this
+        L{TrackingFilePath} that have had their C{open()} method called.
+        """
+        return [path for path in self.trackingList if path.openedFiles]
+
+
+    def clonePath(self, name):
+        """
+        Override L{filepath.FilePath.clonePath} to give the new path a reference
+        to the same tracking list.
+        """
+        clone = TrackingFilePath(name, trackingList=self.trackingList)
+        self.trackingList.append(clone)
+        return clone
+
+
+
+class ExplodingFilePath(filepath.FilePath):
+    """
+    A specialized L{FilePath} which always returns an instance of
+    L{ExplodingFile} from its C{open} method.
+
+    @ivar fp: The L{ExplodingFile} instance most recently returned from the
+        C{open} method.
+    """
+
+    def __init__(self, pathName, originalExploder=None):
+        """
+        Initialize an L{ExplodingFilePath} with a name and a reference to the
+
+        @param pathName: The path name as passed to L{filepath.FilePath}.
+        @type pathName: C{str}
+
+        @param originalExploder: The L{ExplodingFilePath} to associate opened
+        files with.
+        @type originalExploder: L{ExplodingFilePath}
+        """
+        filepath.FilePath.__init__(self, pathName)
+        if originalExploder is None:
+            originalExploder = self
+        self._originalExploder = originalExploder
+
+
+    def open(self, mode=None):
+        """
+        Create, save, and return a new C{ExplodingFile}.
+
+        @param mode: Present for signature compatibility.  Ignored.
+
+        @return: A new C{ExplodingFile}.
+        """
+        f = self._originalExploder.fp = ExplodingFile()
+        return f
+
+
+    def clonePath(self, name):
+        return ExplodingFilePath(name, self._originalExploder)
+
+
+
 class FilePathTestCase(AbstractFilePathTestCase):
+    """
+    Test various L{FilePath} path manipulations.
+    """
 
     def test_chmod(self):
         """
@@ -454,6 +619,64 @@ class FilePathTestCase(AbstractFilePathTestCase):
         self.failUnlessEqual(content, newcontent)
 
 
+    def test_getContentFileClosing(self):
+        """
+        If reading from the underlying file raises an exception,
+        L{FilePath.getContent} raises that exception after closing the file.
+        """
+        fp = ExplodingFilePath("")
+        self.assertRaises(IOError, fp.getContent)
+        self.assertTrue(fp.fp.closed)
+
+
+    def test_setContentFileClosing(self):
+        """
+        If writing to the underlying file raises an exception,
+        L{FilePath.setContent} raises that exception after closing the file.
+        """
+        fp = ExplodingFilePath("")
+        self.assertRaises(IOError, fp.setContent, "blah")
+        self.assertTrue(fp.fp.closed)
+
+
+    def test_setContentNameCollision(self):
+        """
+        L{FilePath.setContent} will use a different temporary filename on each
+        invocation, so that multiple processes, threads, or reentrant
+        invocations will not collide with each other.
+        """
+        fp = TrackingFilePath(self.mktemp())
+        fp.setContent("alpha")
+        fp.setContent("beta")
+
+        # Sanity check: setContent should only open one derivative path each
+        # time to store the temporary file.
+        openedSiblings = fp.openedPaths()
+        self.assertEquals(len(openedSiblings), 2)
+        self.assertNotEquals(openedSiblings[0], openedSiblings[1])
+
+
+    def test_setContentExtension(self):
+        """
+        L{FilePath.setContent} creates temporary files with a user-supplied
+        extension, so that if it is somehow interrupted while writing them, the
+        file that it leaves behind will be identifiable.
+        """
+        fp = TrackingFilePath(self.mktemp())
+        fp.setContent("hello")
+        opened = fp.openedPaths()
+        self.assertEquals(len(opened), 1)
+        self.assertTrue(opened[0].basename().endswith(".new"),
+                        "%s does not end with default '.new' extension" % (
+                            opened[0].basename()))
+        fp.setContent("goodbye", "-something-else")
+        opened = fp.openedPaths()
+        self.assertEquals(len(opened), 2)
+        self.assertTrue(opened[1].basename().endswith("-something-else"),
+                        "%s does not end with -something-else extension" % (
+                            opened[1].basename()))
+
+
     def test_symbolicLink(self):
         """
         Verify the behavior of the C{isLink} method against links and
@@ -594,6 +817,16 @@ class FilePathTestCase(AbstractFilePathTestCase):
         self.failIf(filepath.FilePath('z') !=
                     filepath.FilePath('z'))
 
+
+    def test_descendantOnly(self):
+        """
+        If C{".."} is in the sequence passed to L{FilePath.descendant},
+        L{InsecurePath} is raised.
+        """
+        self.assertRaises(
+            filepath.InsecurePath, self.path.descendant, ['a', '..'])
+
+
     def testSibling(self):
         p = self.path.child('sibling_start')
         ts = p.sibling('sibling_test')
@@ -609,7 +842,24 @@ class FilePathTestCase(AbstractFilePathTestCase):
         ts.createDirectory()
         self.assertIn(ts, self.path.parent().children())
 
-    def testRemove(self):
+
+    def test_temporarySiblingExtension(self):
+        """
+        If L{FilePath.temporarySibling} is given an extension argument, it will
+        produce path objects with that extension appended to their names.
+        """
+        testExtension = ".test-extension"
+        ts = self.path.temporarySibling(testExtension)
+        self.assertTrue(ts.basename().endswith(testExtension),
+                        "%s does not end with %s" % (
+                            ts.basename(), testExtension))
+
+
+    def test_removeDirectory(self):
+        """
+        L{FilePath.remove} on a L{FilePath} that refers to a directory will
+        recursively delete its contents.
+        """
         self.path.remove()
         self.failIf(self.path.exists())
 
@@ -627,8 +877,11 @@ class FilePathTestCase(AbstractFilePathTestCase):
         self.assertTrue(self.path.child("sub1").exists())
 
 
-    def test_copyTo(self):
-        self.assertRaises((OSError, IOError), self.path.copyTo, self.path.child('file1'))
+    def test_copyToDirectory(self):
+        """
+        L{FilePath.copyTo} makes a copy of all the contents of the directory
+        named by that L{FilePath} if it is able to do so.
+        """
         oldPaths = list(self.path.walk()) # Record initial state
         fp = filepath.FilePath(self.mktemp())
         self.path.copyTo(fp)
@@ -640,6 +893,49 @@ class FilePathTestCase(AbstractFilePathTestCase):
         self.assertEquals(newPaths, oldPaths)
 
 
+    def test_copyToMissingDestFileClosing(self):
+        """
+        If an exception is raised while L{FilePath.copyTo} is trying to open
+        source file to read from, the destination file is closed and the
+        exception is raised to the caller of L{FilePath.copyTo}.
+        """
+        nosuch = self.path.child("nothere")
+        # Make it look like something to copy, even though it doesn't exist.
+        # This could happen if the file is deleted between the isfile check and
+        # the file actually being opened.
+        nosuch.isfile = lambda: True
+
+        # We won't get as far as writing to this file, but it's still useful for
+        # tracking whether we closed it.
+        destination = ExplodingFilePath(self.mktemp())
+
+        self.assertRaises(IOError, nosuch.copyTo, destination)
+        self.assertTrue(destination.fp.closed)
+
+
+    def test_copyToFileClosing(self):
+        """
+        If an exception is raised while L{FilePath.copyTo} is copying bytes
+        between two regular files, the source and destination files are closed
+        and the exception propagates to the caller of L{FilePath.copyTo}.
+        """
+        destination = ExplodingFilePath(self.mktemp())
+        source = ExplodingFilePath(__file__)
+        self.assertRaises(IOError, source.copyTo, destination)
+        self.assertTrue(source.fp.closed)
+        self.assertTrue(destination.fp.closed)
+
+
+    def test_copyToDirectoryItself(self):
+        """
+        L{FilePath.copyTo} fails with an OSError or IOError (depending on
+        platform, as it propagates errors from open() and write()) when
+        attempting to copy a directory to a child of itself.
+        """
+        self.assertRaises((OSError, IOError),
+                          self.path.copyTo, self.path.child('file1'))
+
+
     def test_copyToWithSymlink(self):
         """
         Verify that copying with followLinks=True copies symlink targets
@@ -681,6 +977,76 @@ class FilePathTestCase(AbstractFilePathTestCase):
         self.assertEquals(newPaths, oldPaths)
 
 
+    def test_moveToExistsCache(self):
+        """
+        A L{FilePath} that has been moved aside with L{FilePath.moveTo} no
+        longer registers as existing.  Its previously non-existent target
+        exists, though, as it was created by the call to C{moveTo}.
+        """
+        fp = filepath.FilePath(self.mktemp())
+        fp2 = filepath.FilePath(self.mktemp())
+        fp.touch()
+
+        # Both a sanity check (make sure the file status looks right) and an
+        # enticement for stat-caching logic to kick in and remember that these
+        # exist / don't exist.
+        self.assertEquals(fp.exists(), True)
+        self.assertEquals(fp2.exists(), False)
+
+        fp.moveTo(fp2)
+        self.assertEqual(fp.exists(), False)
+        self.assertEqual(fp2.exists(), True)
+
+
+    def test_moveToExistsCacheCrossMount(self):
+        """
+        The assertion of test_moveToExistsCache should hold in the case of a
+        cross-mount move.
+        """
+        self.setUpFaultyRename()
+        self.test_moveToExistsCache()
+
+
+    def test_moveToSizeCache(self, hook=lambda : None):
+        """
+        L{FilePath.moveTo} clears its destination's status cache, such that
+        calls to L{FilePath.getsize} after the call to C{moveTo} will report the
+        new size, not the old one.
+
+        This is a separate test from C{test_moveToExistsCache} because it is
+        intended to cover the fact that the destination's cache is dropped;
+        test_moveToExistsCache doesn't cover this case because (currently) a
+        file that doesn't exist yet does not cache the fact of its non-
+        existence.
+        """
+        fp = filepath.FilePath(self.mktemp())
+        fp2 = filepath.FilePath(self.mktemp())
+        fp.setContent("1234")
+        fp2.setContent("1234567890")
+        hook()
+
+        # Sanity check / kick off caching.
+        self.assertEqual(fp.getsize(), 4)
+        self.assertEqual(fp2.getsize(), 10)
+        # Actually attempting to replace a file on Windows would fail with
+        # ERROR_ALREADY_EXISTS, but we don't need to test that, just the cached
+        # metadata, so, delete the file ...
+        os.remove(fp2.path)
+        # ... but don't clear the status cache, as fp2.remove() would.
+        self.assertEqual(fp2.getsize(), 10)
+
+        fp.moveTo(fp2)
+        self.assertEqual(fp2.getsize(), 4)
+
+
+    def test_moveToSizeCacheCrossMount(self):
+        """
+        The assertion of test_moveToSizeCache should hold in the case of a
+        cross-mount move.
+        """
+        self.test_moveToSizeCache(hook=self.setUpFaultyRename)
+
+
     def test_moveToError(self):
         """
         Verify error behavior of moveTo: it should raises one of OSError or
@@ -757,6 +1123,24 @@ class FilePathTestCase(AbstractFilePathTestCase):
         self.assertTrue(invokedWith)
 
 
+    def test_createBinaryMode(self):
+        """
+        L{FilePath.create} should always open (and write to) files in binary
+        mode; line-feed octets should be unmodified.
+
+        (While this test should pass on all platforms, it is only really
+        interesting on platforms which have the concept of binary mode, i.e.
+        Windows platforms.)
+        """
+        path = filepath.FilePath(self.mktemp())
+        f = path.create()
+        self.failUnless("b" in f.mode)
+        f.write("\n")
+        f.close()
+        read = open(path.path, "rb").read()
+        self.assertEqual(read, "\n")
+
+
     def testOpen(self):
         # Opening a file for reading when it does not already exist is an error
         nonexistent = self.path.child('nonexistent')
@@ -846,6 +1230,39 @@ class FilePathTestCase(AbstractFilePathTestCase):
         self.assertRaises((OSError, IOError), existent.open)
 
 
+    def test_openWithExplicitBinaryMode(self):
+        """
+        Due to a bug in Python 2.7 on Windows including multiple 'b'
+        characters in the mode passed to the built-in open() will cause an
+        error.  FilePath.open() ensures that only a single 'b' character is
+        included in the mode passed to the built-in open().
+
+        See http://bugs.python.org/issue7686 for details about the bug.
+        """
+        writer = self.path.child('explicit-binary')
+        file = writer.open('wb')
+        file.write('abc\ndef')
+        file.close()
+        self.assertTrue(writer.exists)
+
+
+    def test_openWithRedundantExplicitBinaryModes(self):
+        """
+        Due to a bug in Python 2.7 on Windows including multiple 'b'
+        characters in the mode passed to the built-in open() will cause an
+        error.  No matter how many 'b' modes are specified, FilePath.open()
+        ensures that only a single 'b' character is included in the mode
+        passed to the built-in open().
+
+        See http://bugs.python.org/issue7686 for details about the bug.
+        """
+        writer = self.path.child('multiple-binary')
+        file = writer.open('wbb')
+        file.write('abc\ndef')
+        file.close()
+        self.assertTrue(writer.exists)
+
+
     def test_existsCache(self):
         """
         Check that C{filepath.FilePath.exists} correctly restat the object if
@@ -858,6 +1275,94 @@ class FilePathTestCase(AbstractFilePathTestCase):
         self.assertEquals(fp.exists(), True)
 
 
+    def test_changed(self):
+        """
+        L{FilePath.changed} indicates that the L{FilePath} has changed, but does
+        not re-read the status information from the filesystem until it is
+        queried again via another method, such as C{getsize}.
+        """
+        fp = filepath.FilePath(self.mktemp())
+        fp.setContent("12345")
+        self.assertEquals(fp.getsize(), 5)
+
+        # Someone else comes along and changes the file.
+        fObj = open(fp.path, 'wb')
+        fObj.write("12345678")
+        fObj.close()
+
+        # Sanity check for caching: size should still be 5.
+        self.assertEquals(fp.getsize(), 5)
+        fp.changed()
+
+        # This path should look like we don't know what status it's in, not that
+        # we know that it didn't exist when last we checked.
+        self.assertEqual(fp.statinfo, None)
+        self.assertEquals(fp.getsize(), 8)
+
+
+    def test_statinfoBitsNotImplementedInWindows(self):
+        """
+        Verify that certain file stats are not available on Windows
+        """
+        self.assertRaises(NotImplementedError, self.path.getInodeNumber)
+        self.assertRaises(NotImplementedError, self.path.getDevice)
+        self.assertRaises(NotImplementedError, self.path.getNumberOfHardLinks)
+        self.assertRaises(NotImplementedError, self.path.getUserID)
+        self.assertRaises(NotImplementedError, self.path.getGroupID)
+
+
+    def test_statinfoBitsAreNumbers(self):
+        """
+        Verify that file inode/device/nlinks/uid/gid stats are numbers in
+        a POSIX environment
+        """
+        c = self.path.child('file1')
+        for p in self.path, c:
+            self.assertIsInstance(p.getInodeNumber(), long)
+            self.assertIsInstance(p.getDevice(), long)
+            self.assertIsInstance(p.getNumberOfHardLinks(), int)
+            self.assertIsInstance(p.getUserID(), int)
+            self.assertIsInstance(p.getGroupID(), int)
+        self.assertEquals(self.path.getUserID(), c.getUserID())
+        self.assertEquals(self.path.getGroupID(), c.getGroupID())
+
+
+    def test_statinfoNumbersAreValid(self):
+        """
+        Verify that the right numbers come back from the right accessor methods
+        for file inode/device/nlinks/uid/gid (in a POSIX environment)
+        """
+        # specify fake statinfo information
+        class FakeStat:
+            st_ino = 200
+            st_dev = 300
+            st_nlink = 400
+            st_uid = 500
+            st_gid = 600
+
+        # monkey patch in a fake restat method for self.path
+        fake = FakeStat()
+        def fakeRestat(*args, **kwargs):
+            self.path.statinfo = fake
+        self.path.restat = fakeRestat
+
+        # ensure that restat will need to be called to get values
+        self.path.statinfo = None
+
+        self.assertEquals(self.path.getInodeNumber(), fake.st_ino)
+        self.assertEquals(self.path.getDevice(), fake.st_dev)
+        self.assertEquals(self.path.getNumberOfHardLinks(), fake.st_nlink)
+        self.assertEquals(self.path.getUserID(), fake.st_uid)
+        self.assertEquals(self.path.getGroupID(), fake.st_gid)
+
+
+    if platform.isWindows():
+        test_statinfoBitsAreNumbers.skip = True
+        test_statinfoNumbersAreValid.skip = True
+    else:
+        test_statinfoBitsNotImplementedInWindows.skip = True
+
+
 
 from twisted.python import urlpath
 
