@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -9,20 +9,20 @@ only specific tests for old API.
 """
 
 # issue1195 TODOs: replace pump.pump() with something involving Deferreds.
-# Clean up warning suppression. Find a better replacement for the handful of
-# reactor.callLater(0.1, ..) calls.
+# Clean up warning suppression.
 
 import sys, os, time, gc
 
 from cStringIO import StringIO
 from zope.interface import implements, Interface
 
+from twisted.python.versions import Version
 from twisted.trial import unittest
-
 from twisted.spread import pb, util, publish, jelly
-from twisted.internet import protocol, main, reactor, defer
+from twisted.internet import protocol, main, reactor
 from twisted.internet.error import ConnectionRefusedError
-from twisted.internet.defer import Deferred, gatherResults
+from twisted.internet.defer import Deferred, gatherResults, succeed
+from twisted.protocols.policies import WrappingFactory
 from twisted.python import failure, log
 from twisted.cred.error import UnauthorizedLogin, UnhandledCredentials
 from twisted.cred import portal, checkers, credentials
@@ -37,17 +37,21 @@ class Dummy(pb.Viewable):
 
 
 class DummyPerspective(pb.Avatar):
+    """
+    An L{IPerspective} avatar which will be used in some tests.
+    """
     def perspective_getDummyViewPoint(self):
         return Dummy()
 
 
+
 class DummyRealm(object):
     implements(portal.IRealm)
 
     def requestAvatar(self, avatarId, mind, *interfaces):
         for iface in interfaces:
             if iface is pb.IPerspective:
-                return iface, DummyPerspective(), lambda: None
+                return iface, DummyPerspective(avatarId), lambda: None
 
 
 class IOPump:
@@ -142,7 +146,7 @@ class SimpleCopy(pb.Copyable):
 class SimpleLocalCopy(pb.RemoteCopy):
     pass
 
-pb.setCopierForClass(SimpleCopy, SimpleLocalCopy)
+pb.setUnjellyableForClass(SimpleCopy, SimpleLocalCopy)
 
 
 class SimpleFactoryCopy(pb.Copyable):
@@ -173,7 +177,7 @@ def createFactoryCopy(state):
         raise RuntimeError("factory method found no object with id")
     return inst
 
-pb.setFactoryForClass(SimpleFactoryCopy, createFactoryCopy)
+pb.setUnjellyableFactoryForClass(SimpleFactoryCopy, createFactoryCopy)
 
 
 class NestedCopy(pb.Referenceable):
@@ -230,7 +234,7 @@ class RatherBaroqueCache(pb.RemoteCache):
     def observe_end(self):
         log.msg("the end of things")
 
-pb.setCopierForClass(VeryVeryComplicatedCacheable, RatherBaroqueCache)
+pb.setUnjellyableForClass(VeryVeryComplicatedCacheable, RatherBaroqueCache)
 
 
 class SimpleLocalCache(pb.RemoteCache):
@@ -246,7 +250,7 @@ class SimpleLocalCache(pb.RemoteCache):
     def check(self):
         return 1
 
-pb.setCopierForClass(SimpleCache, SimpleLocalCache)
+pb.setUnjellyableForClass(SimpleCache, SimpleLocalCache)
 
 
 class NestedCache(pb.Referenceable):
@@ -287,7 +291,11 @@ class DeferredRemote(pb.Referenceable):
         assert 0, "shouldn't have been run!"
 
     def remote_doItLater(self):
-        d = defer.Deferred()
+        """
+        Return a L{Deferred} to be fired on client side. When fired,
+        C{self.runMe} is called.
+        """
+        d = Deferred()
         d.addCallbacks(self.runMe, self.dontRunMe)
         self.d = d
         return d
@@ -329,7 +337,7 @@ class NewStyleCacheCopy(pb.Cacheable, pb.RemoteCache, object):
     def getStateToCacheAndObserveFor(self, perspective, observer):
         return self.__dict__
 
-pb.setCopierForClass(NewStyleCacheCopy, NewStyleCacheCopy)
+pb.setUnjellyableForClass(NewStyleCacheCopy, NewStyleCacheCopy)
 
 
 class Echoer(pb.Root):
@@ -349,7 +357,9 @@ class NewStyleTestCase(unittest.TestCase):
         """
         Create a pb server using L{Echoer} protocol and connect a client to it.
         """
-        self.server = reactor.listenTCP(0, pb.PBServerFactory(Echoer()))
+        self.serverFactory = pb.PBServerFactory(Echoer())
+        self.wrapper = WrappingFactory(self.serverFactory)
+        self.server = reactor.listenTCP(0, self.wrapper)
         clientFactory = pb.PBClientFactory()
         reactor.connectTCP("localhost", self.server.getHost().port,
                            clientFactory)
@@ -367,6 +377,9 @@ class NewStyleTestCase(unittest.TestCase):
         NewStyleCopy2.initialized = 0
         NewStyleCopy2.value = 1
         self.ref.broker.transport.loseConnection()
+        # Disconnect any server-side connections too.
+        for proto in self.wrapper.protocols:
+            proto.transport.loseConnection()
         return self.server.stopListening()
 
     def test_newStyle(self):
@@ -406,14 +419,14 @@ class NewStyleTestCase(unittest.TestCase):
 class ConnectionNotifyServerFactory(pb.PBServerFactory):
     """
     A server factory which stores the last connection and fires a
-    L{defer.Deferred} on connection made. This factory can handle only one
+    L{Deferred} on connection made. This factory can handle only one
     client connection.
 
     @ivar protocolInstance: the last protocol instance.
     @type protocolInstance: C{pb.Broker}
 
     @ivar connectionMade: the deferred fired upon connection.
-    @type connectionMade: C{defer.Deferred}
+    @type connectionMade: C{Deferred}
     """
     protocolInstance = None
 
@@ -422,7 +435,7 @@ class ConnectionNotifyServerFactory(pb.PBServerFactory):
         Initialize the factory.
         """
         pb.PBServerFactory.__init__(self, root)
-        self.connectionMade = defer.Deferred()
+        self.connectionMade = Deferred()
 
 
     def clientConnectionMade(self, protocol):
@@ -430,7 +443,9 @@ class ConnectionNotifyServerFactory(pb.PBServerFactory):
         Store the protocol and fire the connection deferred.
         """
         self.protocolInstance = protocol
-        self.connectionMade.callback(None)
+        d, self.connectionMade = self.connectionMade, None
+        if d is not None:
+            d.callback(None)
 
 
 
@@ -451,7 +466,7 @@ class NewStyleCachedTestCase(unittest.TestCase):
             self.ref = ref
         d1 = clientFactory.getRootObject().addCallback(gotRoot)
         d2 = self.server.factory.connectionMade
-        return defer.gatherResults([d1, d2])
+        return gatherResults([d1, d2])
 
 
     def tearDown(self):
@@ -629,7 +644,7 @@ class BrokerTestCase(unittest.TestCase):
         self.assertIn(rluid, s.localObjects)
         del self.nestedRemote
         # nudge the gc
-        if sys.hexversion >= 0x2000000 and os.name != "java":
+        if sys.hexversion >= 0x2000000:
             gc.collect()
         # try to nudge the GC even if we can't really
         pump.pump()
@@ -692,7 +707,7 @@ class BrokerTestCase(unittest.TestCase):
         pump.flush()
         # del vcc.observer
         # nudge the gc
-        if sys.hexversion >= 0x2000000 and os.name != "java":
+        if sys.hexversion >= 0x2000000:
             gc.collect()
         # try to nudge the GC even if we can't really
         pump.flush()
@@ -921,7 +936,7 @@ class GetPublisher(pb.Referenceable):
         return self.pub
 
 
-pb.setCopierForClass(DumbPublishable, DumbPub)
+pb.setUnjellyableForClass(DumbPublishable, DumbPub)
 
 class DisconnectionTestCase(unittest.TestCase):
     """
@@ -1019,40 +1034,27 @@ class LocalRemoteTest(util.LocalAsRemote):
 
 
 
-class TestRealm(object):
-    """
-    A realm which repeatedly gives out a single instance of L{MyPerspective}
-    for non-anonymous logins and which gives out a new instance of L{Echoer}
-    for each anonymous login.
-    """
-
-    def __init__(self):
-        self.p = MyPerspective()
-
-    def requestAvatar(self, avatarId, mind, interface):
-        """
-        Verify that the mind and interface supplied have the expected values
-        (this should really be done somewhere else, like inside a test method)
-        and return an avatar appropriate for the given identifier.
-        """
-        assert interface == pb.IPerspective
-        assert mind == "BRAINS!"
-        if avatarId is checkers.ANONYMOUS:
-            return pb.IPerspective, Echoer(), lambda: None
-        else:
-            self.p.loggedIn = 1
-            return pb.IPerspective, self.p, self.p.logout
-
-
-
 class MyPerspective(pb.Avatar):
+    """
+    @ivar loggedIn: set to C{True} when the avatar is logged in.
+    @type loggedIn: C{bool}
 
+    @ivar loggedOut: set to C{True} when the avatar is logged out.
+    @type loggedOut: C{bool}
+    """
     implements(pb.IPerspective)
 
     loggedIn = loggedOut = False
 
-    def __init__(self):
-        pass
+    def __init__(self, avatarId):
+        self.avatarId = avatarId
+
+
+    def perspective_getAvatarId(self):
+        """
+        Return the avatar identifier which was used to access this avatar.
+        """
+        return self.avatarId
 
 
     def perspective_getViewPoint(self):
@@ -1073,6 +1075,41 @@ class MyPerspective(pb.Avatar):
 
 
 
+class TestRealm(object):
+    """
+    A realm which repeatedly gives out a single instance of L{MyPerspective}
+    for non-anonymous logins and which gives out a new instance of L{Echoer}
+    for each anonymous login.
+
+    @ivar lastPerspective: The L{MyPerspective} most recently created and
+        returned from C{requestAvatar}.
+
+    @ivar perspectiveFactory: A one-argument callable which will be used to
+        create avatars to be returned from C{requestAvatar}.
+    """
+    perspectiveFactory = MyPerspective
+
+    lastPerspective = None
+
+    def requestAvatar(self, avatarId, mind, interface):
+        """
+        Verify that the mind and interface supplied have the expected values
+        (this should really be done somewhere else, like inside a test method)
+        and return an avatar appropriate for the given identifier.
+        """
+        assert interface == pb.IPerspective
+        assert mind == "BRAINS!"
+        if avatarId is checkers.ANONYMOUS:
+            return pb.IPerspective, Echoer(), lambda: None
+        else:
+            self.lastPerspective = self.perspectiveFactory(avatarId)
+            self.lastPerspective.loggedIn = True
+            return (
+                pb.IPerspective, self.lastPerspective,
+                self.lastPerspective.logout)
+
+
+
 class MyView(pb.Viewable):
 
     def view_check(self, user):
@@ -1092,7 +1129,7 @@ class NewCredTestCase(unittest.TestCase):
         """
         self.realm = TestRealm()
         self.portal = portal.Portal(self.realm)
-        self.factory = pb.PBServerFactory(self.portal)
+        self.factory = ConnectionNotifyServerFactory(self.portal)
         self.port = reactor.listenTCP(0, self.factory, interface="127.0.0.1")
         self.portno = self.port.getHost().port
 
@@ -1103,10 +1140,21 @@ class NewCredTestCase(unittest.TestCase):
         """
         return self.port.stopListening()
 
+
     def getFactoryAndRootObject(self, clientFactory=pb.PBClientFactory):
+        """
+        Create a connection to the test server.
+
+        @param clientFactory: the factory class used to create the connection.
+
+        @return: a tuple (C{factory}, C{deferred}), where factory is an
+            instance of C{clientFactory} and C{deferred} the L{Deferred} firing
+            with the PB root object.
+        """
         factory = clientFactory()
         rootObjDeferred = factory.getRootObject()
-        reactor.connectTCP('127.0.0.1', self.portno, factory)
+        connector = reactor.connectTCP('127.0.0.1', self.portno, factory)
+        self.addCleanup(connector.disconnect)
         return factory, rootObjDeferred
 
 
@@ -1118,8 +1166,8 @@ class NewCredTestCase(unittest.TestCase):
         factory, rootObjDeferred = self.getFactoryAndRootObject()
 
         def gotRootObject(rootObj):
-            self.failUnless(isinstance(rootObj, pb.RemoteReference))
-            disconnectedDeferred = defer.Deferred()
+            self.assertIsInstance(rootObj, pb.RemoteReference)
+            disconnectedDeferred = Deferred()
             rootObj.notifyOnDisconnect(disconnectedDeferred.callback)
             factory.disconnect()
             return disconnectedDeferred
@@ -1135,7 +1183,7 @@ class NewCredTestCase(unittest.TestCase):
         factory, rootObjDeferred = self.getFactoryAndRootObject()
 
         def gotRootObject(rootObj):
-            disconnectedDeferred = defer.Deferred()
+            disconnectedDeferred = Deferred()
             rootObj.notifyOnDisconnect(disconnectedDeferred.callback)
 
             def lostConnection(ign):
@@ -1171,7 +1219,7 @@ class NewCredTestCase(unittest.TestCase):
         def gotRootObject(rootObj):
             self.assertIsInstance(rootObj, pb.RemoteReference)
 
-            d = defer.Deferred()
+            d = Deferred()
             rootObj.notifyOnDisconnect(d.callback)
             factory.disconnect()
 
@@ -1181,7 +1229,7 @@ class NewCredTestCase(unittest.TestCase):
                 def gotAnotherRootObject(anotherRootObj):
                     self.assertIsInstance(anotherRootObj, pb.RemoteReference)
 
-                    d = defer.Deferred()
+                    d = Deferred()
                     anotherRootObj.notifyOnDisconnect(d.callback)
                     factory.disconnect()
                     return d
@@ -1216,6 +1264,19 @@ class NewCredTestCase(unittest.TestCase):
         return self.assertFailure(loginDeferred, ConnectionRefusedError)
 
 
+    def _disconnect(self, ignore, factory):
+        """
+        Helper method disconnecting the given client factory and returning a
+        C{Deferred} that will fire when the server connection has noticed the
+        disconnection.
+        """
+        disconnectedDeferred = Deferred()
+        self.factory.protocolInstance.notifyOnDisconnect(
+            lambda: disconnectedDeferred.callback(None))
+        factory.disconnect()
+        return disconnectedDeferred
+
+
     def test_loginLogout(self):
         """
         Test that login can be performed with IUsernamePassword credentials and
@@ -1235,20 +1296,89 @@ class NewCredTestCase(unittest.TestCase):
 
         d = factory.login(creds, mind)
         def cbLogin(perspective):
-            self.assertEquals(self.realm.p.loggedIn, 1)
-            self.assert_(isinstance(perspective, pb.RemoteReference))
-
-            factory.disconnect()
-            d = Deferred()
-            reactor.callLater(1.0, d.callback, None)
-            return d
+            self.assertTrue(self.realm.lastPerspective.loggedIn)
+            self.assertIsInstance(perspective, pb.RemoteReference)
+            return self._disconnect(None, factory)
         d.addCallback(cbLogin)
 
         def cbLogout(ignored):
-            self.assertEquals(self.realm.p.loggedOut, 1)
+            self.assertTrue(self.realm.lastPerspective.loggedOut)
         d.addCallback(cbLogout)
 
-        reactor.connectTCP("127.0.0.1", self.portno, factory)
+        connector = reactor.connectTCP("127.0.0.1", self.portno, factory)
+        self.addCleanup(connector.disconnect)
+        return d
+
+
+    def test_logoutAfterDecref(self):
+        """
+        If a L{RemoteReference} to an L{IPerspective} avatar is decrefed and
+        there remain no other references to the avatar on the server, the
+        avatar is garbage collected and the logout method called.
+        """
+        loggedOut = Deferred()
+
+        class EventPerspective(pb.Avatar):
+            """
+            An avatar which fires a Deferred when it is logged out.
+            """
+            def __init__(self, avatarId):
+                pass
+
+            def logout(self):
+                loggedOut.callback(None)
+
+        self.realm.perspectiveFactory = EventPerspective
+
+        self.portal.registerChecker(
+            checkers.InMemoryUsernamePasswordDatabaseDontUse(foo='bar'))
+        factory = pb.PBClientFactory()
+        d = factory.login(
+            credentials.UsernamePassword('foo', 'bar'), "BRAINS!")
+        def cbLoggedIn(avatar):
+            # Just wait for the logout to happen, as it should since the
+            # reference to the avatar will shortly no longer exists.
+            return loggedOut
+        d.addCallback(cbLoggedIn)
+        def cbLoggedOut(ignored):
+            # Verify that the server broker's _localCleanup dict isn't growing
+            # without bound.
+            self.assertEqual(self.factory.protocolInstance._localCleanup, {})
+        d.addCallback(cbLoggedOut)
+        d.addCallback(self._disconnect, factory)
+        connector = reactor.connectTCP("127.0.0.1", self.portno, factory)
+        self.addCleanup(connector.disconnect)
+        return d
+
+
+    def test_concurrentLogin(self):
+        """
+        Two different correct login attempts can be made on the same root
+        object at the same time and produce two different resulting avatars.
+        """
+        self.portal.registerChecker(
+            checkers.InMemoryUsernamePasswordDatabaseDontUse(
+                foo='bar', baz='quux'))
+        factory = pb.PBClientFactory()
+
+        firstLogin = factory.login(
+            credentials.UsernamePassword('foo', 'bar'), "BRAINS!")
+        secondLogin = factory.login(
+            credentials.UsernamePassword('baz', 'quux'), "BRAINS!")
+        d = gatherResults([firstLogin, secondLogin])
+        def cbLoggedIn((first, second)):
+            return gatherResults([
+                    first.callRemote('getAvatarId'),
+                    second.callRemote('getAvatarId')])
+        d.addCallback(cbLoggedIn)
+        def cbAvatarIds((first, second)):
+            self.assertEqual(first, 'foo')
+            self.assertEqual(second, 'baz')
+        d.addCallback(cbAvatarIds)
+        d.addCallback(self._disconnect, factory)
+
+        connector = reactor.connectTCP('127.0.0.1', self.portno, factory)
+        self.addCleanup(connector.disconnect)
         return d
 
 
@@ -1270,13 +1400,14 @@ class NewCredTestCase(unittest.TestCase):
         self.assertFailure(secondLogin, UnauthorizedLogin)
         d = gatherResults([firstLogin, secondLogin])
 
-        def cleanup(passthrough):
-            self.flushLoggedErrors(UnauthorizedLogin)
-            factory.disconnect()
-            return passthrough
-        d.addBoth(cleanup)
+        def cleanup(ignore):
+            errors = self.flushLoggedErrors(UnauthorizedLogin)
+            self.assertEquals(len(errors), 2)
+            return self._disconnect(None, factory)
+        d.addCallback(cleanup)
 
-        reactor.connectTCP("127.0.0.1", self.portno, factory)
+        connector = reactor.connectTCP("127.0.0.1", self.portno, factory)
+        self.addCleanup(connector.disconnect)
         return d
 
 
@@ -1296,14 +1427,10 @@ class NewCredTestCase(unittest.TestCase):
 
         d.addCallback(self.assertEqual, 123)
 
-        def cleanup(passthrough):
-            factory.disconnect()
-            d = Deferred()
-            reactor.callLater(1.0, d.callback, None)
-            return d
-        d.addBoth(cleanup)
+        d.addCallback(self._disconnect, factory)
 
-        reactor.connectTCP("127.0.0.1", self.portno, factory)
+        connector = reactor.connectTCP("127.0.0.1", self.portno, factory)
+        self.addCleanup(connector.disconnect)
         return d
 
 
@@ -1315,16 +1442,17 @@ class NewCredTestCase(unittest.TestCase):
         self.portal.registerChecker(
             checkers.InMemoryUsernamePasswordDatabaseDontUse(user='pass'))
         factory = pb.PBClientFactory()
-        d = factory.login(credentials.Anonymous(),"BRAINS!")
+        d = factory.login(credentials.Anonymous(), "BRAINS!")
         self.assertFailure(d, UnhandledCredentials)
 
-        def cleanup(passthrough):
-            self.flushLoggedErrors(UnhandledCredentials)
-            factory.disconnect()
-            return passthrough
-        d.addBoth(cleanup)
+        def cleanup(ignore):
+            errors = self.flushLoggedErrors(UnhandledCredentials)
+            self.assertEquals(len(errors), 1)
+            return self._disconnect(None, factory)
+        d.addCallback(cleanup)
 
-        reactor.connectTCP('127.0.0.1', self.portno, factory)
+        connector = reactor.connectTCP('127.0.0.1', self.portno, factory)
+        self.addCleanup(connector.disconnect)
         return d
 
 
@@ -1345,12 +1473,10 @@ class NewCredTestCase(unittest.TestCase):
 
         d.addCallback(self.assertEqual, 123)
 
-        def cleanup(passthrough):
-            factory.disconnect()
-            return passthrough
-        d.addBoth(cleanup)
+        d.addCallback(self._disconnect, factory)
 
-        reactor.connectTCP('127.0.0.1', self.portno, factory)
+        connector = reactor.connectTCP('127.0.0.1', self.portno, factory)
+        self.addCleanup(connector.disconnect)
         return d
 
 
@@ -1372,12 +1498,10 @@ class NewCredTestCase(unittest.TestCase):
 
         d.addCallback(self.assertEqual, 123)
 
-        def cleanup(passthrough):
-            factory.disconnect()
-            return passthrough
-        d.addBoth(cleanup)
+        d.addCallback(self._disconnect, factory)
 
-        reactor.connectTCP('127.0.0.1', self.portno, factory)
+        connector = reactor.connectTCP('127.0.0.1', self.portno, factory)
+        self.addCleanup(connector.disconnect)
         return d
 
 
@@ -1400,16 +1524,12 @@ class NewCredTestCase(unittest.TestCase):
             return viewpoint.callRemote("check")
         d.addCallback(cbView)
 
-        d.addCallback(self.failUnless)
+        d.addCallback(self.assertTrue)
 
-        def cleanup(passthrough):
-            factory.disconnect()
-            d = Deferred()
-            reactor.callLater(1.0, d.callback, None)
-            return d
-        d.addBoth(cleanup)
+        d.addCallback(self._disconnect, factory)
 
-        reactor.connectTCP("127.0.0.1", self.portno, factory)
+        connector = reactor.connectTCP("127.0.0.1", self.portno, factory)
+        self.addCleanup(connector.disconnect)
         return d
 
 
@@ -1417,6 +1537,9 @@ class NewCredTestCase(unittest.TestCase):
 class NonSubclassingPerspective:
     implements(pb.IPerspective)
 
+    def __init__(self, avatarId):
+        pass
+
     # IPerspective implementation
     def perspectiveMessageReceived(self, broker, message, args, kwargs):
         args = broker.unserialize(args, self)
@@ -1430,21 +1553,28 @@ class NonSubclassingPerspective:
 
 
 class NSPTestCase(unittest.TestCase):
+    """
+    Tests for authentication against a realm where the L{IPerspective}
+    implementation is not a subclass of L{Avatar}.
+    """
     def setUp(self):
         self.realm = TestRealm()
-        self.realm.p = NonSubclassingPerspective()
+        self.realm.perspectiveFactory = NonSubclassingPerspective
         self.portal = portal.Portal(self.realm)
         self.checker = checkers.InMemoryUsernamePasswordDatabaseDontUse()
         self.checker.addUser("user", "pass")
         self.portal.registerChecker(self.checker)
-        self.factory = pb.PBServerFactory(self.portal)
+        self.factory = WrappingFactory(pb.PBServerFactory(self.portal))
         self.port = reactor.listenTCP(0, self.factory, interface="127.0.0.1")
+        self.addCleanup(self.port.stopListening)
         self.portno = self.port.getHost().port
 
-    def tearDown(self):
-        return self.port.stopListening()
 
     def test_NSP(self):
+        """
+        An L{IPerspective} implementation which does not subclass
+        L{Avatar} can expose remote methods for the client to call.
+        """
         factory = pb.PBClientFactory()
         d = factory.login(credentials.UsernamePassword('user', 'pass'),
                           "BRAINS!")
@@ -1452,7 +1582,11 @@ class NSPTestCase(unittest.TestCase):
         d.addCallback(lambda p: p.callRemote('ANYTHING', 'here', bar='baz'))
         d.addCallback(self.assertEquals,
                       ('ANYTHING', ('here',), {'bar': 'baz'}))
-        d.addCallback(lambda res: factory.disconnect())
+        def cleanup(ignored):
+            factory.disconnect()
+            for p in self.factory.protocols:
+                p.transport.loseConnection()
+        d.addCallback(cleanup)
         return d
 
 
@@ -1503,7 +1637,7 @@ class Forwarded:
         """
         Asynchronously return C{True}.
         """
-        return defer.succeed(True)
+        return succeed(True)
 
 
 class SpreadUtilTestCase(unittest.TestCase):
@@ -1527,7 +1661,7 @@ class SpreadUtilTestCase(unittest.TestCase):
         o = LocalRemoteTest()
         o = LocalRemoteTest()
         d = o.callRemote("add", 2, y=4)
-        self.assertTrue(isinstance(d, defer.Deferred))
+        self.assertIsInstance(d, Deferred)
         d.addCallback(self.assertEquals, 6)
         return d
 
@@ -1535,7 +1669,6 @@ class SpreadUtilTestCase(unittest.TestCase):
         """
         Test a asynchronous failure on a remote method call.
         """
-        l = []
         o = LocalRemoteTest()
         d = o.callRemote("fail")
         def eb(f):
@@ -1616,3 +1749,26 @@ class PBWithSecurityOptionsTest(unittest.TestCase):
         broker = factory.buildProtocol(None)
         self.assertIdentical(broker.security, security)
 
+
+
+class DeprecationTests(unittest.TestCase):
+    """
+    Tests for certain deprecations of free-functions in L{twisted.spread.pb}.
+    """
+    def test_noOperationDeprecated(self):
+        """
+        L{pb.noOperation} is deprecated.
+        """
+        self.callDeprecated(
+            Version("twisted", 8, 2, 0),
+            pb.noOperation, 1, 2, x=3, y=4)
+
+
+    def test_printTraceback(self):
+        """
+        L{pb.printTraceback} is deprecated.
+        """
+        self.callDeprecated(
+            Version("twisted", 8, 2, 0),
+            pb.printTraceback,
+            "printTraceback deprecation fake traceback value")
